@@ -165,11 +165,10 @@ enum Command {
 
     /// Publish the project to the Hex package manager
     ///
-    /// This command uses this environment variables:
+    /// This command uses the environment variable:
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
     /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
+    ///
     #[command(verbatim_doc_comment)]
     Publish {
         #[arg(long)]
@@ -314,6 +313,12 @@ pub enum ExportTarget {
         /// The path to write the JSON file to
         output: Utf8PathBuf,
     },
+    /// Package information (gleam.toml) in JSON format
+    PackageInformation {
+        #[arg(long = "out", required = true)]
+        /// The path to write the JSON file to
+        output: Utf8PathBuf,
+    },
 }
 
 #[derive(Args, Debug, Clone)]
@@ -390,11 +395,10 @@ enum Dependencies {
 enum Hex {
     /// Retire a release from Hex
     ///
-    /// This command uses this environment variables:
+    /// This command uses the environment variable:
+    ///
+    /// - HEXPM_API_KEY: (optional) A Hex API key to authenticate against the Hex package manager.
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
-    /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
     #[command(verbatim_doc_comment)]
     Retire {
         package: String,
@@ -409,21 +413,19 @@ enum Hex {
 
     /// Un-retire a release from Hex
     ///
-    /// This command uses this environment variables:
+    /// This command uses this environment variable:
+    ///
+    /// - HEXPM_API_KEY: (optional) A Hex API key to authenticate against the Hex package manager.
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
-    /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
     #[command(verbatim_doc_comment)]
     Unretire { package: String, version: String },
 
     /// Revert a release from Hex
     ///
-    /// This command uses this environment variables:
+    /// This command uses this environment variable:
+    ///
+    /// - HEXPM_API_KEY: (optional) A Hex API key to authenticate against the Hex package manager.
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
-    /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
     #[command(verbatim_doc_comment)]
     Revert {
         #[arg(long)]
@@ -451,21 +453,19 @@ enum Docs {
 
     /// Publish HTML docs to HexDocs
     ///
-    /// This command uses this environment variables:
+    /// This command uses this environment variable:
+    ///
+    /// - HEXPM_API_KEY: (optional) A Hex API key to authenticate against the Hex package manager.
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
-    /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
     #[command(verbatim_doc_comment)]
     Publish,
 
     /// Remove HTML docs from HexDocs
     ///
-    /// This command uses this environment variables:
+    /// This command uses this environment variable:
+    ///
+    /// - HEXPM_API_KEY: (optional) A Hex API key to authenticate against the Hex package manager.
     ///
-    /// - HEXPM_USER: (optional) The Hex username to authenticate with.
-    /// - HEXPM_PASS: (optional) The Hex password to authenticate with.
-    /// - HEXPM_API_KEY: (optional) A Hex API key to use instead of authenticating.
     #[command(verbatim_doc_comment)]
     Remove {
         /// The name of the package
@@ -663,6 +663,10 @@ fn parse_and_run_command() -> Result<(), Error> {
             let paths = find_project_paths()?;
             export::package_interface(&paths, output)
         }
+        Command::Export(ExportTarget::PackageInformation { output }) => {
+            let paths = find_project_paths()?;
+            export::package_information(&paths, output)
+        }
     }
 }
 
