@@ -20,15 +20,16 @@ use crate::{
     dep_tree,
     line_numbers::LineNumbers,
     parse::SpannedString,
+    reference::{EntityKind, ReferenceKind},
     type_::{
         self, AccessorsMap, Deprecation, ModuleInterface, Opaque, PatternConstructor,
-        RecordAccessor, Type, TypeAliasConstructor, TypeConstructor, TypeValueConstructor,
-        TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
+        RecordAccessor, References, Type, TypeAliasConstructor, TypeConstructor,
+        TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
         ValueConstructorVariant, Warning,
         environment::*,
         error::{Error, FeatureKind, MissingAnnotation, Named, Problems, convert_unify_error},
         expression::{ExprTyper, FunctionDefinition, Implementations},
-        fields::{FieldMap, FieldMapBuilder},
+        fields::FieldMapBuilder,
         hydrator::Hydrator,
         prelude::*,
     },
@@ -202,7 +203,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         let env = Environment::new(
             self.ids.clone(),
             self.package_config.name.clone(),
-            self.package_config.gleam_version.clone(),
+            self.package_config
+                .gleam_version
+                .clone()
+                .map(|version| version.as_pubgrub()),
             self.module_name.clone(),
             self.target,
             self.importable_modules,
@@ -340,6 +344,15 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 type_aliases,
                 documentation,
                 contains_echo: echo_found,
+                references: References {
+                    imported_modules: env
+                        .imported_modules
+                        .values()
+                        .map(|(_location, module)| module.name.clone())
+                        .collect(),
+                    value_references: env.references.value_references,
+                    type_references: env.references.type_references,
+                },
             },
             names: type_names,
         };
@@ -372,6 +385,8 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         } = c;
         self.check_name_case(name_location, &name, Named::Constant);
 
+        environment.references.begin_constant();
+
         let definition = FunctionDefinition {
             has_body: true,
             has_erlang_external: false,
@@ -407,6 +422,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 location,
                 literal: typed_expr.clone(),
                 module: self.module_name.clone(),
+                name: name.clone(),
                 implementations,
             },
             type_: type_.clone(),
@@ -421,14 +437,17 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         );
         environment.insert_module_value(name.clone(), variant);
 
-        if publicity.is_private() {
-            environment.init_usage(
-                name.clone(),
-                EntityKind::PrivateConstant,
-                location,
-                &mut self.problems,
-            );
-        }
+        environment
+            .references
+            .register_constant(name.clone(), location, publicity);
+
+        environment.references.register_value_reference(
+            environment.current_module.clone(),
+            name.clone(),
+            &name,
+            name_location,
+            ReferenceKind::Definition,
+        );
 
         Definition::ModuleConstant(ModuleConstant {
             documentation: doc,
@@ -516,6 +535,11 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             .map(|(a, t)| a.set_type(t.clone()))
             .collect_vec();
 
+        // We have already registered the function in the `register_value_from_function`
+        // method, but here we must set this as the current function again, so that anything
+        // we reference in the body of it can be tracked properly in the call graph.
+        environment.references.set_current_node(name.clone());
+
         // Infer the type using the preregistered args + return types as a starting point
         let result = environment.in_new_scope(&mut self.problems, |environment, problems| {
             let mut expr_typer = ExprTyper::new(environment, definition, problems);
@@ -634,6 +658,14 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             deprecation.clone(),
         );
 
+        environment.references.register_value_reference(
+            environment.current_module.clone(),
+            name.clone(),
+            &name,
+            name_location,
+            ReferenceKind::Definition,
+        );
+
         Definition::Function(Function {
             documentation: doc,
             location,
@@ -964,25 +996,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             .type_
             .clone();
 
-        let Accessors {
-            shared_accessors,
-            variant_specific_accessors,
-        } = custom_type_accessors(constructors, &mut hydrator, environment, &mut self.problems)?;
-
-        let map = AccessorsMap {
-            publicity: if *opaque {
-                Publicity::Private
-            } else {
-                *publicity
-            },
-            shared_accessors,
-            // TODO: improve the ownership here so that we can use the
-            // `return_type_constructor` below rather than looking it up twice.
-            type_: type_.clone(),
-            variant_specific_accessors,
-        };
-        environment.insert_accessors(name.clone(), map);
-
         let mut constructors_data = vec![];
 
         let mut index = 0;
@@ -996,12 +1009,35 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 continue;
             }
 
-            let mut field_map = FieldMap::new(constructor.arguments.len() as u32);
+            // If the constructor belongs to an opaque type then it's going to be
+            // considered as private.
+            let value_constructor_publicity = if *opaque {
+                Publicity::Private
+            } else {
+                *publicity
+            };
+
+            environment.references.register_value(
+                constructor.name.clone(),
+                EntityKind::Constructor,
+                constructor.location,
+                value_constructor_publicity,
+            );
+
+            environment
+                .references
+                .register_type_reference_in_call_graph(name.clone());
+
+            let mut field_map_builder = FieldMapBuilder::new(constructor.arguments.len() as u32);
             let mut args_types = Vec::with_capacity(constructor.arguments.len());
             let mut fields = Vec::with_capacity(constructor.arguments.len());
 
-            for (i, RecordConstructorArg { label, ast, .. }) in
-                constructor.arguments.iter().enumerate()
+            for RecordConstructorArg {
+                label,
+                ast,
+                location,
+                ..
+            } in constructor.arguments.iter()
             {
                 // Build a type from the annotation AST
                 let t = match hydrator.type_from_ast(ast, environment, &mut self.problems) {
@@ -1020,17 +1056,17 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 // Register the type for this parameter
                 args_types.push(t);
 
-                // Register the label for this parameter, if there is one
-                if let Some((location, label)) = label {
-                    if field_map.insert(label.clone(), i as u32).is_err() {
-                        self.problems.error(Error::DuplicateField {
-                            label: label.clone(),
-                            location: *location,
-                        });
-                    };
+                let (label_location, label) = match label {
+                    Some((location, label)) => (*location, Some(label)),
+                    None => (*location, None),
+                };
+
+                // Register the label for this parameter
+                if let Err(error) = field_map_builder.add(label, label_location) {
+                    self.problems.error(error);
                 }
             }
-            let field_map = field_map.into_option();
+            let field_map = field_map_builder.finish();
             // Insert constructor function into module scope
             let mut type_ = type_.deref().clone();
             type_.set_custom_type_variant(index as u16);
@@ -1053,14 +1089,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             };
             index += 1;
 
-            // If the contructor belongs to an opaque type then it's going to be
-            // considered as private.
-            let value_constructor_publicity = if *opaque {
-                Publicity::Private
-            } else {
-                *publicity
-            };
-
             // If the whole custom type is deprecated all of its varints are too.
             // Otherwise just the varint(s) attributed as deprecated are.
             let deprecate_constructor = if deprecation.is_deprecated() {
@@ -1079,14 +1107,13 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 },
             );
 
-            if value_constructor_publicity.is_private() {
-                environment.init_usage(
-                    constructor.name.clone(),
-                    EntityKind::PrivateTypeConstructor(name.clone()),
-                    constructor.location,
-                    &mut self.problems,
-                );
-            }
+            environment.references.register_value_reference(
+                environment.current_module.clone(),
+                constructor.name.clone(),
+                &constructor.name,
+                constructor.name_location,
+                ReferenceKind::Definition,
+            );
 
             constructors_data.push(TypeValueConstructor {
                 name: constructor.name.clone(),
@@ -1111,6 +1138,25 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             );
         }
 
+        let Accessors {
+            shared_accessors,
+            variant_specific_accessors,
+        } = custom_type_accessors(&constructors_data)?;
+
+        let map = AccessorsMap {
+            publicity: if *opaque {
+                Publicity::Private
+            } else {
+                *publicity
+            },
+            shared_accessors,
+            // TODO: improve the ownership here so that we can use the
+            // `return_type_constructor` below rather than looking it up twice.
+            type_: type_.clone(),
+            variant_specific_accessors,
+        };
+        environment.insert_accessors(name.clone(), map);
+
         let opaque = if *opaque {
             Opaque::Opaque
         } else {
@@ -1203,20 +1249,24 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             name.clone(),
         );
 
+        environment
+            .references
+            .register_type(name.clone(), EntityKind::Type, *location, publicity);
+
+        environment.references.register_type_reference(
+            environment.current_module.clone(),
+            name.clone(),
+            name,
+            *name_location,
+            ReferenceKind::Definition,
+        );
+
         if *opaque && constructors.is_empty() {
             self.problems.warning(Warning::OpaqueExternalType {
                 location: *location,
             });
         }
 
-        if publicity.is_private() {
-            environment.init_usage(
-                name.clone(),
-                EntityKind::PrivateType,
-                *location,
-                &mut self.problems,
-            );
-        };
         Ok(())
     }
 
@@ -1243,6 +1293,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
 
         self.check_name_case(*name_location, name, Named::TypeAlias);
 
+        environment
+            .references
+            .register_type(name.clone(), EntityKind::Type, *location, *publicity);
+
         // Use the hydrator to convert the AST into a type, erroring if the AST was invalid
         // in some fashion.
         let mut hydrator = Hydrator::new();
@@ -1294,16 +1348,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         };
         let result = tryblock();
         self.record_if_error(result);
-
-        // Register the type for detection of dead code.
-        if publicity.is_private() {
-            environment.init_usage(
-                name.clone(),
-                EntityKind::PrivateType,
-                *location,
-                &mut self.problems,
-            );
-        };
     }
 
     fn make_type_vars(
@@ -1359,6 +1403,13 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
 
         self.check_name_case(*name_location, name, Named::Function);
 
+        environment.references.register_value(
+            name.clone(),
+            EntityKind::Function,
+            *location,
+            *publicity,
+        );
+
         let mut builder = FieldMapBuilder::new(args.len() as u32);
         for Arg {
             names, location, ..
@@ -1408,14 +1459,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             *publicity,
             deprecation.clone(),
         );
-        if publicity.is_private() {
-            environment.init_usage(
-                name.clone(),
-                EntityKind::PrivateFunction,
-                *location,
-                &mut self.problems,
-            );
-        };
         Ok(())
     }
 
@@ -1624,6 +1667,7 @@ fn generalise_module_constant(
         literal: *value.clone(),
         module: module_name.clone(),
         implementations,
+        name: name.clone(),
     };
     environment.insert_variable(
         name.clone(),
@@ -1759,46 +1803,39 @@ struct Accessors {
     variant_specific_accessors: Vec<HashMap<EcoString, RecordAccessor>>,
 }
 
-fn custom_type_accessors<A: std::fmt::Debug>(
-    constructors: &[RecordConstructor<A>],
-    hydrator: &mut Hydrator,
-    environment: &mut Environment<'_>,
-    problems: &mut Problems,
-) -> Result<Accessors, Error> {
+fn custom_type_accessors(constructors: &[TypeValueConstructor]) -> Result<Accessors, Error> {
     let args = get_compatible_record_fields(constructors);
 
     let mut shared_accessors = HashMap::with_capacity(args.len());
 
-    hydrator.disallow_new_type_variables();
-    for (index, label, ast) in args {
-        let type_ = hydrator.type_from_ast(ast, environment, problems)?;
+    for (index, label, type_) in args {
         let _ = shared_accessors.insert(
             label.clone(),
             RecordAccessor {
                 index: index as u64,
                 label: label.clone(),
-                type_,
+                type_: type_.clone(),
             },
         );
     }
 
-    let mut variant_specific_accessors = Vec::with_capacity(constructors.len());
+    let mut variant_specific_accessors: Vec<HashMap<EcoString, RecordAccessor>> =
+        Vec::with_capacity(constructors.len());
 
     for constructor in constructors {
-        let mut fields = HashMap::with_capacity(constructor.arguments.len());
+        let mut fields = HashMap::with_capacity(constructor.parameters.len());
 
-        for (index, argument) in constructor.arguments.iter().enumerate() {
-            let Some((_location, label)) = &argument.label else {
+        for (index, parameter) in constructor.parameters.iter().enumerate() {
+            let Some(label) = &parameter.label else {
                 continue;
             };
 
-            let type_ = hydrator.type_from_ast(&argument.ast, environment, problems)?;
             let _ = fields.insert(
                 label.clone(),
                 RecordAccessor {
                     index: index as u64,
                     label: label.clone(),
-                    type_,
+                    type_: parameter.type_.clone(),
                 },
             );
         }
@@ -1813,9 +1850,9 @@ fn custom_type_accessors<A: std::fmt::Debug>(
 
 /// Returns the fields that have the same label and type across all variants of
 /// the given type.
-fn get_compatible_record_fields<A: std::fmt::Debug>(
-    constructors: &[RecordConstructor<A>],
-) -> Vec<(usize, &EcoString, &TypeAst)> {
+fn get_compatible_record_fields(
+    constructors: &[TypeValueConstructor],
+) -> Vec<(usize, &EcoString, &Arc<Type>)> {
     let mut compatible = vec![];
 
     let first = match constructors.first() {
@@ -1823,10 +1860,10 @@ fn get_compatible_record_fields<A: std::fmt::Debug>(
         None => return compatible,
     };
 
-    'next_argument: for (index, first_argument) in first.arguments.iter().enumerate() {
+    'next_argument: for (index, first_parameter) in first.parameters.iter().enumerate() {
         // Fields without labels do not have accessors
-        let first_label = match first_argument.label.as_ref() {
-            Some((_, label)) => label,
+        let first_label = match first_parameter.label.as_ref() {
+            Some(label) => label,
             None => continue 'next_argument,
         };
 
@@ -1834,22 +1871,22 @@ fn get_compatible_record_fields<A: std::fmt::Debug>(
         // with the same label and the same type
         for constructor in constructors.iter().skip(1) {
             // The field must exist in all variants
-            let argument = match constructor.arguments.get(index) {
+            let parameter = match constructor.parameters.get(index) {
                 Some(argument) => argument,
                 None => continue 'next_argument,
             };
 
             // The labels must be the same
-            if argument
+            if parameter
                 .label
                 .as_ref()
-                .is_none_or(|(_, arg_label)| arg_label != first_label)
+                .is_none_or(|arg_label| arg_label != first_label)
             {
                 continue 'next_argument;
             }
 
             // The types must be the same
-            if !argument.ast.is_logically_equal(&first_argument.ast) {
+            if !parameter.type_.same_as(&first_parameter.type_) {
                 continue 'next_argument;
             }
         }
@@ -1857,7 +1894,7 @@ fn get_compatible_record_fields<A: std::fmt::Debug>(
         // The previous loop did not find any incompatible fields in the other
         // variants so this field is compatible across variants and we should
         // generate an accessor for it.
-        compatible.push((index, first_label, &first_argument.ast))
+        compatible.push((index, first_label, &first_parameter.type_))
     }
 
     compatible
@@ -1893,9 +1930,9 @@ fn get_type_dependencies(type_: &TypeAst) -> Vec<EcoString> {
             }
             deps.extend(get_type_dependencies(return_))
         }
-        TypeAst::Tuple(TypeAstTuple { elems, .. }) => {
-            for elem in elems {
-                deps.extend(get_type_dependencies(elem))
+        TypeAst::Tuple(TypeAstTuple { elements, .. }) => {
+            for element in elements {
+                deps.extend(get_type_dependencies(element))
             }
         }
     }
