@@ -12,7 +12,7 @@ pub use self::untyped::{FunctionLiteralKind, UntypedExpr};
 pub use self::constant::{Constant, TypedConstant, UntypedConstant};
 
 use crate::analyse::Inferred;
-use crate::build::{Located, Target};
+use crate::build::{Located, Target, module_erlang_name};
 use crate::parse::SpannedString;
 use crate::type_::error::VariableOrigin;
 use crate::type_::expression::Implementations;
@@ -34,6 +34,7 @@ pub const RECORD_UPDATE_VARIABLE: &str = "_record";
 pub const ASSERT_FAIL_VARIABLE: &str = "_assert_fail";
 pub const ASSERT_SUBJECT_VARIABLE: &str = "_assert_subject";
 pub const CAPTURE_VARIABLE: &str = "_capture";
+pub const BLOCK_VARIABLE: &str = "_block";
 
 pub trait HasLocation {
     fn location(&self) -> SrcSpan;
@@ -52,6 +53,12 @@ pub struct Module<Info, Statements> {
     pub names: Names,
 }
 
+impl<Info, Statements> Module<Info, Statements> {
+    pub fn erlang_name(&self) -> EcoString {
+        module_erlang_name(&self.name)
+    }
+}
+
 impl TypedModule {
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         self.definitions
@@ -269,6 +276,7 @@ impl<T: PartialEq> RecordConstructorArg<T> {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct TypeAstConstructor {
     pub location: SrcSpan,
+    pub name_location: SrcSpan,
     pub module: Option<(EcoString, SrcSpan)>,
     pub name: EcoString,
     pub arguments: Vec<TypeAst>,
@@ -290,7 +298,7 @@ pub struct TypeAstVar {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct TypeAstTuple {
     pub location: SrcSpan,
-    pub elems: Vec<TypeAst>,
+    pub elements: Vec<TypeAst>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -326,12 +334,14 @@ impl TypeAst {
                 name,
                 arguments,
                 location: _,
+                name_location: _,
             }) => match other {
                 TypeAst::Constructor(TypeAstConstructor {
                     module: o_module,
                     name: o_name,
                     arguments: o_arguments,
                     location: _,
+                    name_location: _,
                 }) => {
                     let module_name =
                         |m: &Option<(EcoString, _)>| m.as_ref().map(|(m, _)| m.clone());
@@ -371,15 +381,18 @@ impl TypeAst {
                 }) => name == o_name,
                 _ => false,
             },
-            TypeAst::Tuple(TypeAstTuple { elems, location: _ }) => match other {
+            TypeAst::Tuple(TypeAstTuple {
+                elements,
+                location: _,
+            }) => match other {
                 TypeAst::Tuple(TypeAstTuple {
-                    elems: o_elems,
+                    elements: other_elements,
                     location: _,
                 }) => {
-                    elems.len() == o_elems.len()
-                        && elems
+                    elements.len() == other_elements.len()
+                        && elements
                             .iter()
-                            .zip(o_elems)
+                            .zip(other_elements)
                             .all(|a| a.0.is_logically_equal(a.1))
                 }
                 _ => false,
@@ -418,7 +431,7 @@ impl TypeAst {
 
                     None
                 })
-                .or(Some(Located::Annotation(self.location(), type_))),
+                .or(Some(Located::Annotation { ast: self, type_ })),
             TypeAst::Constructor(TypeAstConstructor {
                 arguments, module, ..
             }) => type_
@@ -445,11 +458,11 @@ impl TypeAst {
                         None
                     }
                 }))
-                .or(Some(Located::Annotation(self.location(), type_))),
-            TypeAst::Tuple(TypeAstTuple { elems, .. }) => type_
+                .or(Some(Located::Annotation { ast: self, type_ })),
+            TypeAst::Tuple(TypeAstTuple { elements, .. }) => type_
                 .tuple_types()
                 .and_then(|elem_types| {
-                    if let Some(e) = elems
+                    if let Some(e) = elements
                         .iter()
                         .zip(elem_types)
                         .find_map(|(e, e_type)| e.find_node(byte_index, e_type.clone()))
@@ -459,8 +472,8 @@ impl TypeAst {
 
                     None
                 })
-                .or(Some(Located::Annotation(self.location(), type_))),
-            TypeAst::Var(_) | TypeAst::Hole(_) => Some(Located::Annotation(self.location(), type_)),
+                .or(Some(Located::Annotation { ast: self, type_ })),
+            TypeAst::Var(_) | TypeAst::Hole(_) => Some(Located::Annotation { ast: self, type_ }),
         }
     }
 
@@ -471,9 +484,9 @@ impl TypeAst {
             TypeAst::Hole(hole) => buffer.push_str(&hole.name),
             TypeAst::Tuple(tuple) => {
                 buffer.push_str("#(");
-                for (i, elem) in tuple.elems.iter().enumerate() {
-                    elem.print(buffer);
-                    if i < tuple.elems.len() - 1 {
+                for (i, element) in tuple.elements.iter().enumerate() {
+                    element.print(buffer);
+                    if i < tuple.elements.len() - 1 {
                         buffer.push_str(", ");
                     }
                 }
@@ -543,6 +556,7 @@ fn type_ast_print_constructor() {
         name: "SomeType".into(),
         module: Some(("some_module".into(), SrcSpan { start: 1, end: 1 })),
         location: SrcSpan { start: 1, end: 1 },
+        name_location: SrcSpan { start: 1, end: 1 },
         arguments: vec![
             TypeAst::Var(TypeAstVar {
                 location: SrcSpan { start: 1, end: 1 },
@@ -563,11 +577,12 @@ fn type_ast_print_tuple() {
     let mut buffer = EcoString::new();
     let ast = TypeAst::Tuple(TypeAstTuple {
         location: SrcSpan { start: 1, end: 1 },
-        elems: vec![
+        elements: vec![
             TypeAst::Constructor(TypeAstConstructor {
                 name: "SomeType".into(),
                 module: Some(("some_module".into(), SrcSpan { start: 1, end: 1 })),
                 location: SrcSpan { start: 1, end: 1 },
+                name_location: SrcSpan { start: 1, end: 1 },
                 arguments: vec![
                     TypeAst::Var(TypeAstVar {
                         location: SrcSpan { start: 1, end: 1 },
@@ -711,7 +726,7 @@ impl<T> Import<T> {
         match self.as_name.as_ref() {
             Some((AssignName::Variable(name), _)) => Some(name.clone()),
             Some((AssignName::Discard(_), _)) => None,
-            None => self.module.split('/').last().map(EcoString::from),
+            None => self.module.split('/').next_back().map(EcoString::from),
         }
     }
 
@@ -892,20 +907,21 @@ impl TypedDefinition {
 
             Definition::CustomType(custom) => {
                 // Check if location is within the type of one of the arguments of a constructor.
-                if let Some(annotation) = custom
+                if let Some(constructor) = custom
                     .constructors
                     .iter()
                     .find(|constructor| constructor.location.contains(byte_index))
-                    .and_then(|constructor| {
-                        constructor
-                            .arguments
-                            .iter()
-                            .find(|arg| arg.location.contains(byte_index))
-                    })
-                    .filter(|arg| arg.location.contains(byte_index))
-                    .and_then(|arg| arg.ast.find_node(byte_index, arg.type_.clone()))
                 {
-                    return Some(annotation);
+                    if let Some(annotation) = constructor
+                        .arguments
+                        .iter()
+                        .find(|arg| arg.location.contains(byte_index))
+                        .and_then(|arg| arg.ast.find_node(byte_index, arg.type_.clone()))
+                    {
+                        return Some(annotation);
+                    }
+
+                    return Some(Located::VariantConstructorDefinition(constructor));
                 }
 
                 // Note that the custom type `.location` covers the function
@@ -1077,6 +1093,8 @@ impl<A, B, C, E> Definition<A, B, C, E> {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct UnqualifiedImport {
     pub location: SrcSpan,
+    /// The location excluding the potential `as ...` clause, or the `type` keyword
+    pub imported_name_location: SrcSpan,
     pub name: EcoString,
     pub as_name: Option<EcoString>,
 }
@@ -1147,6 +1165,8 @@ pub enum OperatorKind {
     StringConcatenation,
 }
 
+pub const PIPE_PRECEDENCE: u8 = 6;
+
 impl BinOp {
     pub fn precedence(&self) -> u8 {
         // Ensure that this matches the other precedence function for guards
@@ -1229,6 +1249,90 @@ impl BinOp {
     pub fn can_be_grouped_with(&self, other: &BinOp) -> bool {
         self.operator_kind() == other.operator_kind()
     }
+
+    pub(crate) fn is_float_operator(&self) -> bool {
+        match self {
+            BinOp::LtFloat
+            | BinOp::LtEqFloat
+            | BinOp::GtEqFloat
+            | BinOp::GtFloat
+            | BinOp::AddFloat
+            | BinOp::SubFloat
+            | BinOp::MultFloat
+            | BinOp::DivFloat => true,
+
+            BinOp::And
+            | BinOp::Or
+            | BinOp::Eq
+            | BinOp::NotEq
+            | BinOp::LtInt
+            | BinOp::LtEqInt
+            | BinOp::GtEqInt
+            | BinOp::GtInt
+            | BinOp::AddInt
+            | BinOp::SubInt
+            | BinOp::MultInt
+            | BinOp::DivInt
+            | BinOp::RemainderInt
+            | BinOp::Concatenate => false,
+        }
+    }
+
+    pub(crate) fn is_int_operator(&self) -> bool {
+        match self {
+            BinOp::LtInt
+            | BinOp::LtEqInt
+            | BinOp::GtEqInt
+            | BinOp::GtInt
+            | BinOp::AddInt
+            | BinOp::SubInt
+            | BinOp::MultInt
+            | BinOp::DivInt
+            | BinOp::RemainderInt => true,
+
+            BinOp::And
+            | BinOp::Or
+            | BinOp::Eq
+            | BinOp::NotEq
+            | BinOp::LtFloat
+            | BinOp::LtEqFloat
+            | BinOp::GtEqFloat
+            | BinOp::GtFloat
+            | BinOp::AddFloat
+            | BinOp::SubFloat
+            | BinOp::MultFloat
+            | BinOp::DivFloat
+            | BinOp::Concatenate => false,
+        }
+    }
+
+    pub fn float_equivalent(&self) -> Option<BinOp> {
+        match self {
+            BinOp::LtInt => Some(BinOp::LtFloat),
+            BinOp::LtEqInt => Some(BinOp::LtEqFloat),
+            BinOp::GtEqInt => Some(BinOp::GtEqFloat),
+            BinOp::GtInt => Some(BinOp::GtFloat),
+            BinOp::AddInt => Some(BinOp::AddFloat),
+            BinOp::SubInt => Some(BinOp::SubFloat),
+            BinOp::MultInt => Some(BinOp::MultFloat),
+            BinOp::DivInt => Some(BinOp::DivFloat),
+            _ => None,
+        }
+    }
+
+    pub fn int_equivalent(&self) -> Option<BinOp> {
+        match self {
+            BinOp::LtFloat => Some(BinOp::LtInt),
+            BinOp::LtEqFloat => Some(BinOp::LtEqInt),
+            BinOp::GtEqFloat => Some(BinOp::GtEqInt),
+            BinOp::GtFloat => Some(BinOp::GtInt),
+            BinOp::AddFloat => Some(BinOp::AddInt),
+            BinOp::SubFloat => Some(BinOp::SubInt),
+            BinOp::MultFloat => Some(BinOp::MultInt),
+            BinOp::DivFloat => Some(BinOp::DivInt),
+            _ => None,
+        }
+    }
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -1739,6 +1843,17 @@ impl SrcSpan {
         byte_index >= self.start && byte_index <= self.end
     }
 
+    /// Merges two spans into a new one that starts at the start of the smaller
+    /// one and ends at the end of the bigger one. For example:
+    ///
+    /// ```txt
+    /// wibble    wobble
+    /// ─┬────    ─┬────
+    ///  │         ╰─ one span
+    ///  ╰─ the other span
+    /// ─┬──────────────
+    ///  ╰─ the span you get by merging the two
+    /// ```
     pub fn merge(&self, with: &SrcSpan) -> SrcSpan {
         Self {
             start: self.start.min(with.start),
@@ -1819,6 +1934,7 @@ pub enum Pattern<Type> {
     /// The constructor for a custom type. Starts with an uppercase letter.
     Constructor {
         location: SrcSpan,
+        name_location: SrcSpan,
         name: EcoString,
         arguments: Vec<CallArg<Self>>,
         module: Option<(EcoString, SrcSpan)>,
@@ -1829,7 +1945,7 @@ pub enum Pattern<Type> {
 
     Tuple {
         location: SrcSpan,
-        elems: Vec<Self>,
+        elements: Vec<Self>,
     },
 
     BitArray {
@@ -1862,7 +1978,7 @@ impl Default for Inferred<()> {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum AssignName {
     Variable(EcoString),
     Discard(EcoString),
@@ -1985,7 +2101,9 @@ impl TypedPattern {
 
             Pattern::Discard { type_, .. } => type_.clone(),
 
-            Pattern::Tuple { elems, .. } => type_::tuple(elems.iter().map(|p| p.type_()).collect()),
+            Pattern::Tuple { elements, .. } => {
+                type_::tuple(elements.iter().map(|p| p.type_()).collect())
+            }
         }
     }
 
@@ -2018,7 +2136,7 @@ impl TypedPattern {
                 Some(spread_location) if spread_location.contains(byte_index) => {
                     Some(Located::PatternSpread {
                         spread_location: *spread_location,
-                        arguments,
+                        pattern: self,
                     })
                 }
 
@@ -2029,7 +2147,9 @@ impl TypedPattern {
                 .find_map(|p| p.find_node(byte_index))
                 .or_else(|| tail.as_ref().and_then(|p| p.find_node(byte_index))),
 
-            Pattern::Tuple { elems, .. } => elems.iter().find_map(|p| p.find_node(byte_index)),
+            Pattern::Tuple { elements, .. } => {
+                elements.iter().find_map(|p| p.find_node(byte_index))
+            }
 
             Pattern::BitArray { segments, .. } => segments
                 .iter()
@@ -2038,7 +2158,49 @@ impl TypedPattern {
         }
         .or(Some(Located::Pattern(self)))
     }
+
+    /// If the pattern is a `Constructor` with a spread, it returns a tuple with
+    /// all the ignored fields. Split in unlabelled and labelled ones.
+    ///
+    pub(crate) fn unused_arguments(&self) -> Option<PatternUnusedArguments> {
+        let TypedPattern::Constructor {
+            arguments,
+            spread: Some(_),
+            ..
+        } = self
+        else {
+            return None;
+        };
+
+        let mut positional = vec![];
+        let mut labelled = vec![];
+        for argument in arguments {
+            // We only want to display the arguments that were ignored using `..`.
+            // Any argument ignored that way is marked as implicit, so if it is
+            // not implicit we just ignore it.
+            if !argument.is_implicit() {
+                continue;
+            }
+            let type_ = argument.value.type_();
+            match &argument.label {
+                Some(label) => labelled.push((label.clone(), type_)),
+                None => positional.push(type_),
+            }
+        }
+
+        Some(PatternUnusedArguments {
+            positional,
+            labelled,
+        })
+    }
 }
+
+#[derive(Debug, Default)]
+pub struct PatternUnusedArguments {
+    pub positional: Vec<Arc<Type>>,
+    pub labelled: Vec<(EcoString, Arc<Type>)>,
+}
+
 impl<A> HasLocation for Pattern<A> {
     fn location(&self) -> SrcSpan {
         self.location()
@@ -2105,6 +2267,63 @@ pub struct BitArraySegment<Value, Type> {
     pub type_: Type,
 }
 
+#[derive(Debug, PartialEq, Copy, Clone)]
+pub enum Endianness {
+    Big,
+    Little,
+}
+
+impl Endianness {
+    pub fn is_big(&self) -> bool {
+        *self == Endianness::Big
+    }
+}
+
+impl<Value> BitArraySegment<Value, Arc<Type>> {
+    #[must_use]
+    pub fn has_native_option(&self) -> bool {
+        self.options
+            .iter()
+            .any(|x| matches!(x, BitArrayOption::Native { .. }))
+    }
+
+    pub fn endianness(&self) -> Endianness {
+        if self
+            .options
+            .iter()
+            .any(|x| matches!(x, BitArrayOption::Little { .. }))
+        {
+            Endianness::Little
+        } else {
+            Endianness::Big
+        }
+    }
+
+    pub fn size(&self) -> Option<&Value> {
+        self.options.iter().find_map(|x| match x {
+            BitArrayOption::Size { value, .. } => Some(value.as_ref()),
+            _ => None,
+        })
+    }
+
+    pub fn unit(&self) -> u8 {
+        self.options
+            .iter()
+            .find_map(|option| match option {
+                BitArrayOption::Unit { value, .. } => Some(*value),
+                _ => None,
+            })
+            .unwrap_or(1)
+    }
+}
+
+impl<Value, Type> BitArraySegment<Value, Type> {
+    #[must_use]
+    pub(crate) fn has_type_option(&self) -> bool {
+        self.options.iter().any(|option| option.is_type_option())
+    }
+}
+
 impl TypedExprBitArraySegment {
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         self.value.find_node(byte_index)
@@ -2248,6 +2467,29 @@ impl<A> BitArrayOption<A> {
             BitArrayOption::Unit { .. } => "unit".into(),
         }
     }
+
+    fn is_type_option(&self) -> bool {
+        match self {
+            BitArrayOption::Bytes { .. }
+            | BitArrayOption::Int { .. }
+            | BitArrayOption::Float { .. }
+            | BitArrayOption::Bits { .. }
+            | BitArrayOption::Utf8 { .. }
+            | BitArrayOption::Utf16 { .. }
+            | BitArrayOption::Utf32 { .. }
+            | BitArrayOption::Utf8Codepoint { .. }
+            | BitArrayOption::Utf16Codepoint { .. }
+            | BitArrayOption::Utf32Codepoint { .. } => true,
+
+            BitArrayOption::Signed { .. }
+            | BitArrayOption::Unsigned { .. }
+            | BitArrayOption::Big { .. }
+            | BitArrayOption::Little { .. }
+            | BitArrayOption::Native { .. }
+            | BitArrayOption::Size { .. }
+            | BitArrayOption::Unit { .. } => false,
+        }
+    }
 }
 
 impl BitArrayOption<TypedPattern> {
