@@ -77,6 +77,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         name: BinOp,
+        name_location: SrcSpan,
         left: Box<Self>,
         right: Box<Self>,
     },
@@ -90,6 +91,7 @@ pub enum TypedExpr {
 
     RecordAccess {
         location: SrcSpan,
+        field_start: u32,
         type_: Arc<Type>,
         label: EcoString,
         index: u64,
@@ -109,7 +111,7 @@ pub enum TypedExpr {
     Tuple {
         location: SrcSpan,
         type_: Arc<Type>,
-        elems: Vec<Self>,
+        elements: Vec<Self>,
     },
 
     TupleIndex {
@@ -234,7 +236,7 @@ impl TypedExpr {
 
             Self::Echo { expression, .. } => expression
                 .as_ref()
-                .and_then(|e| e.find_node(byte_index))
+                .and_then(|expression| expression.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
             Self::Todo { kind, .. } => match kind {
@@ -281,7 +283,8 @@ impl TypedExpr {
             // if during iteration, an element is encountered with a start index
             // beyond the index under search.
             Self::Tuple {
-                elems: expressions, ..
+                elements: expressions,
+                ..
             } => {
                 for expression in expressions {
                     if expression.location().start > byte_index {
@@ -415,7 +418,8 @@ impl TypedExpr {
             // if during iteration, an element is encountered with a start index
             // beyond the index under search.
             Self::Tuple {
-                elems: expressions, ..
+                elements: expressions,
+                ..
             } => {
                 for expression in expressions {
                     if expression.location().start > byte_index {
@@ -488,7 +492,7 @@ impl TypedExpr {
 
             Self::Echo { expression, .. } => expression
                 .as_ref()
-                .and_then(|e| e.find_statement(byte_index)),
+                .and_then(|expression| expression.find_statement(byte_index)),
 
             Self::BitArray { segments, .. } => segments
                 .iter()
