@@ -194,10 +194,11 @@ pub trait Visit<'ast> {
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
         name: &'ast BinOp,
+        name_location: &'ast SrcSpan,
         left: &'ast TypedExpr,
         right: &'ast TypedExpr,
     ) {
-        visit_typed_expr_bin_op(self, location, type_, name, left, right);
+        visit_typed_expr_bin_op(self, location, type_, name, name_location, left, right);
     }
 
     fn visit_typed_expr_case(
@@ -213,12 +214,13 @@ pub trait Visit<'ast> {
     fn visit_typed_expr_record_access(
         &mut self,
         location: &'ast SrcSpan,
+        field_start: &'ast u32,
         type_: &'ast Arc<Type>,
         label: &'ast EcoString,
         index: &'ast u64,
         record: &'ast TypedExpr,
     ) {
-        visit_typed_expr_record_access(self, location, type_, label, index, record);
+        visit_typed_expr_record_access(self, location, field_start, type_, label, index, record);
     }
 
     #[allow(clippy::too_many_arguments)]
@@ -248,9 +250,9 @@ pub trait Visit<'ast> {
         &mut self,
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
-        elems: &'ast [TypedExpr],
+        elements: &'ast [TypedExpr],
     ) {
-        visit_typed_expr_tuple(self, location, type_, elems);
+        visit_typed_expr_tuple(self, location, type_, elements);
     }
 
     fn visit_typed_expr_tuple_index(
@@ -469,6 +471,7 @@ pub trait Visit<'ast> {
     fn visit_typed_pattern_constructor(
         &mut self,
         location: &'ast SrcSpan,
+        name_location: &'ast SrcSpan,
         name: &'ast EcoString,
         arguments: &'ast Vec<CallArg<TypedPattern>>,
         module: &'ast Option<(EcoString, SrcSpan)>,
@@ -479,6 +482,7 @@ pub trait Visit<'ast> {
         visit_typed_pattern_constructor(
             self,
             location,
+            name_location,
             name,
             arguments,
             module,
@@ -495,9 +499,9 @@ pub trait Visit<'ast> {
     fn visit_typed_pattern_tuple(
         &mut self,
         location: &'ast SrcSpan,
-        elems: &'ast Vec<TypedPattern>,
+        elements: &'ast Vec<TypedPattern>,
     ) {
-        visit_typed_pattern_tuple(self, location, elems);
+        visit_typed_pattern_tuple(self, location, elements);
     }
 
     fn visit_typed_pattern_bit_array(
@@ -539,11 +543,12 @@ pub trait Visit<'ast> {
     fn visit_type_ast_constructor(
         &mut self,
         location: &'ast SrcSpan,
+        name_location: &'ast SrcSpan,
         module: &'ast Option<(EcoString, SrcSpan)>,
         name: &'ast EcoString,
         arguments: &'ast Vec<TypeAst>,
     ) {
-        visit_type_ast_constructor(self, location, module, name, arguments);
+        visit_type_ast_constructor(self, location, name_location, module, name, arguments);
     }
 
     fn visit_type_ast_fn(
@@ -559,8 +564,8 @@ pub trait Visit<'ast> {
         visit_type_ast_var(self, location, name);
     }
 
-    fn visit_type_ast_tuple(&mut self, location: &'ast SrcSpan, elems: &'ast Vec<TypeAst>) {
-        visit_type_ast_tuple(self, location, elems);
+    fn visit_type_ast_tuple(&mut self, location: &'ast SrcSpan, elements: &'ast Vec<TypeAst>) {
+        visit_type_ast_tuple(self, location, elements);
     }
 
     fn visit_type_ast_hole(&mut self, location: &'ast SrcSpan, name: &'ast EcoString) {
@@ -605,11 +610,12 @@ where
     match node {
         TypeAst::Constructor(super::TypeAstConstructor {
             location,
+            name_location,
             arguments,
             module,
             name,
         }) => {
-            v.visit_type_ast_constructor(location, module, name, arguments);
+            v.visit_type_ast_constructor(location, name_location, module, name, arguments);
         }
         TypeAst::Fn(super::TypeAstFn {
             location,
@@ -621,8 +627,8 @@ where
         TypeAst::Var(super::TypeAstVar { location, name }) => {
             v.visit_type_ast_var(location, name);
         }
-        TypeAst::Tuple(super::TypeAstTuple { location, elems }) => {
-            v.visit_type_ast_tuple(location, elems);
+        TypeAst::Tuple(super::TypeAstTuple { location, elements }) => {
+            v.visit_type_ast_tuple(location, elements);
         }
         TypeAst::Hole(super::TypeAstHole { location, name }) => {
             v.visit_type_ast_hole(location, name);
@@ -633,6 +639,7 @@ where
 pub fn visit_type_ast_constructor<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
+    _name_location: &'a SrcSpan,
     _module: &'a Option<(EcoString, SrcSpan)>,
     _name: &'a EcoString,
     arguments: &'a Vec<TypeAst>,
@@ -665,12 +672,12 @@ where
     // No further traversal needed for variables
 }
 
-pub fn visit_type_ast_tuple<'a, V>(v: &mut V, _location: &'a SrcSpan, elems: &'a Vec<TypeAst>)
+pub fn visit_type_ast_tuple<'a, V>(v: &mut V, _location: &'a SrcSpan, elements: &'a Vec<TypeAst>)
 where
     V: Visit<'a> + ?Sized,
 {
-    for elem in elems {
-        v.visit_type_ast(elem);
+    for element in elements {
+        v.visit_type_ast(element);
     }
 }
 
@@ -754,9 +761,10 @@ where
             location,
             type_,
             name,
+            name_location,
             left,
             right,
-        } => v.visit_typed_expr_bin_op(location, type_, name, left, right),
+        } => v.visit_typed_expr_bin_op(location, type_, name, name_location, left, right),
         TypedExpr::Case {
             location,
             type_,
@@ -765,11 +773,12 @@ where
         } => v.visit_typed_expr_case(location, type_, subjects, clauses),
         TypedExpr::RecordAccess {
             location,
+            field_start,
             type_,
             label,
             index,
             record,
-        } => v.visit_typed_expr_record_access(location, type_, label, index, record),
+        } => v.visit_typed_expr_record_access(location, field_start, type_, label, index, record),
         TypedExpr::ModuleSelect {
             location,
             field_start,
@@ -790,8 +799,8 @@ where
         TypedExpr::Tuple {
             location,
             type_,
-            elems,
-        } => v.visit_typed_expr_tuple(location, type_, elems),
+            elements,
+        } => v.visit_typed_expr_tuple(location, type_, elements),
         TypedExpr::TupleIndex {
             location,
             type_,
@@ -966,6 +975,7 @@ pub fn visit_typed_expr_bin_op<'a, V>(
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
     _name: &'a BinOp,
+    _name_location: &'a SrcSpan,
     left: &'a TypedExpr,
     right: &'a TypedExpr,
 ) where
@@ -996,6 +1006,7 @@ pub fn visit_typed_expr_case<'a, V>(
 pub fn visit_typed_expr_record_access<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
+    _field_start: &'a u32,
     _type_: &'a Arc<Type>,
     _label: &'a EcoString,
     _index: &'a u64,
@@ -1011,7 +1022,7 @@ pub fn visit_typed_expr_module_select<'a, V>(
     _v: &mut V,
     _location: &'a SrcSpan,
     _field_start: &'a u32,
-    _typ: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
     _label: &'a EcoString,
     _module_name: &'a EcoString,
     _module_alias: &'a EcoString,
@@ -1026,12 +1037,12 @@ pub fn visit_typed_expr_tuple<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
-    elems: &'a [TypedExpr],
+    elements: &'a [TypedExpr],
 ) where
     V: Visit<'a> + ?Sized,
 {
-    for elem in elems {
-        v.visit_typed_expr(elem);
+    for element in elements {
+        v.visit_typed_expr(element);
     }
 }
 
@@ -1061,7 +1072,7 @@ pub fn visit_typed_expr_todo<'a, V>(
     }
 }
 
-fn visit_typed_expr_echo<'a, V>(
+pub fn visit_typed_expr_echo<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
@@ -1473,6 +1484,7 @@ where
         } => v.visit_typed_pattern_list(location, elements, tail, type_),
         Pattern::Constructor {
             location,
+            name_location,
             name,
             arguments,
             module,
@@ -1481,6 +1493,7 @@ where
             type_,
         } => v.visit_typed_pattern_constructor(
             location,
+            name_location,
             name,
             arguments,
             module,
@@ -1488,7 +1501,7 @@ where
             spread,
             type_,
         ),
-        Pattern::Tuple { location, elems } => v.visit_typed_pattern_tuple(location, elems),
+        Pattern::Tuple { location, elements } => v.visit_typed_pattern_tuple(location, elements),
         Pattern::BitArray { location, segments } => {
             v.visit_typed_pattern_bit_array(location, segments)
         }
@@ -1593,6 +1606,7 @@ pub fn visit_typed_pattern_list<'a, V>(
 pub fn visit_typed_pattern_constructor<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
+    _name_location: &'a SrcSpan,
     _name: &'a EcoString,
     arguments: &'a Vec<CallArg<TypedPattern>>,
     _module: &'a Option<(EcoString, SrcSpan)>,
@@ -1617,12 +1631,12 @@ where
 pub fn visit_typed_pattern_tuple<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
-    elems: &'a Vec<TypedPattern>,
+    elements: &'a Vec<TypedPattern>,
 ) where
     V: Visit<'a> + ?Sized,
 {
-    for elem in elems {
-        v.visit_typed_pattern(elem);
+    for element in elements {
+        v.visit_typed_pattern(element);
     }
 }
 
