@@ -17,8 +17,8 @@ pub use self::project_compiler::{Built, Options, ProjectCompiler};
 pub use self::telemetry::{NullTelemetry, Telemetry};
 
 use crate::ast::{
-    self, CallArg, CustomType, DefinitionLocation, Pattern, TypeAst, TypedArg, TypedDefinition,
-    TypedExpr, TypedFunction, TypedPattern, TypedStatement,
+    self, CallArg, CustomType, DefinitionLocation, TypeAst, TypedArg, TypedDefinition, TypedExpr,
+    TypedFunction, TypedPattern, TypedRecordConstructor, TypedStatement,
 };
 use crate::type_::Type;
 use crate::{
@@ -246,10 +246,14 @@ pub struct Module {
 }
 
 impl Module {
+    pub fn erlang_name(&self) -> EcoString {
+        module_erlang_name(&self.name)
+    }
+
     pub fn compiled_erlang_path(&self) -> Utf8PathBuf {
-        let mut path = self.name.replace("/", "@");
-        path.push_str(".erl");
-        Utf8PathBuf::from(path.as_ref())
+        let mut path = Utf8PathBuf::from(&module_erlang_name(&self.name));
+        assert!(path.set_extension("erl"), "Couldn't set file extension");
+        path
     }
 
     pub fn is_test(&self) -> bool {
@@ -321,6 +325,10 @@ impl Module {
     }
 }
 
+pub fn module_erlang_name(gleam_name: &EcoString) -> EcoString {
+    gleam_name.replace("/", "@")
+}
+
 #[derive(Debug, Clone, PartialEq)]
 pub struct UnqualifiedImport<'a> {
     pub name: &'a EcoString,
@@ -334,14 +342,18 @@ pub enum Located<'a> {
     Pattern(&'a TypedPattern),
     PatternSpread {
         spread_location: SrcSpan,
-        arguments: &'a Vec<CallArg<TypedPattern>>,
+        pattern: &'a TypedPattern,
     },
     Statement(&'a TypedStatement),
     Expression(&'a TypedExpr),
     ModuleStatement(&'a TypedDefinition),
+    VariantConstructorDefinition(&'a TypedRecordConstructor),
     FunctionBody(&'a TypedFunction),
     Arg(&'a TypedArg),
-    Annotation(SrcSpan, std::sync::Arc<Type>),
+    Annotation {
+        ast: &'a TypeAst,
+        type_: std::sync::Arc<Type>,
+    },
     UnqualifiedImport(UnqualifiedImport<'a>),
     Label(SrcSpan, std::sync::Arc<Type>),
     ModuleName {
@@ -382,6 +394,10 @@ impl<'a> Located<'a> {
                 module: None,
                 span: statement.location(),
             }),
+            Self::VariantConstructorDefinition(record) => Some(DefinitionLocation {
+                module: None,
+                span: record.location,
+            }),
             Self::UnqualifiedImport(UnqualifiedImport {
                 module,
                 name,
@@ -401,7 +417,7 @@ impl<'a> Located<'a> {
                 }
             }),
             Self::Arg(_) => None,
-            Self::Annotation(_, type_) => self.type_location(importable_modules, type_.clone()),
+            Self::Annotation { type_, .. } => self.type_location(importable_modules, type_.clone()),
             Self::Label(_, _) => None,
             Self::ModuleName { name, .. } => Some(DefinitionLocation {
                 module: Some((*name).clone()),
@@ -416,10 +432,11 @@ impl<'a> Located<'a> {
             Located::Statement(statement) => Some(statement.type_()),
             Located::Expression(typed_expr) => Some(typed_expr.type_()),
             Located::Arg(arg) => Some(arg.type_.clone()),
-            Located::Label(_, type_) | Located::Annotation(_, type_) => Some(type_.clone()),
+            Located::Label(_, type_) | Located::Annotation { type_, .. } => Some(type_.clone()),
 
             Located::PatternSpread { .. } => None,
             Located::ModuleStatement(definition) => None,
+            Located::VariantConstructorDefinition(_) => None,
             Located::FunctionBody(function) => None,
             Located::UnqualifiedImport(unqualified_import) => None,
             Located::ModuleName { .. } => None,
@@ -484,11 +501,11 @@ fn type_to_definition_locations<'a>(
         // For fn types we just get the locations of their arguments and return
         // type.
         //
-        Type::Fn { args, retrn } => args
+        Type::Fn { args, return_ } => args
             .iter()
             .flat_map(|arg| type_to_definition_locations(arg.clone(), importable_modules))
             .chain(type_to_definition_locations(
-                retrn.clone(),
+                return_.clone(),
                 importable_modules,
             ))
             .collect_vec(),
@@ -505,9 +522,9 @@ fn type_to_definition_locations<'a>(
 
         // In case of tuples we get the locations of the wrapped types.
         //
-        Type::Tuple { elems } => elems
+        Type::Tuple { elements } => elements
             .iter()
-            .flat_map(|elem| type_to_definition_locations(elem.clone(), importable_modules))
+            .flat_map(|element| type_to_definition_locations(element.clone(), importable_modules))
             .collect_vec(),
     }
 }
