@@ -24,7 +24,7 @@ use crate::{
 use askama::Template;
 use ecow::EcoString;
 use itertools::Itertools;
-use serde::Serialize;
+use serde::{Deserialize, Serialize};
 use serde_json::to_string as serde_to_string;
 
 const MAX_COLUMNS: isize = 65;
@@ -35,6 +35,12 @@ pub enum DocContext {
     Build,
 }
 
+#[derive(PartialEq, Debug, Serialize, Deserialize)]
+pub struct PackageInformation {
+    #[serde(rename = "gleam.toml")]
+    package_config: PackageConfig,
+}
+
 pub fn generate_html<IO: FileSystemReader>(
     paths: &ProjectPaths,
     config: &PackageConfig,
@@ -169,15 +175,6 @@ pub fn generate_html<IO: FileSystemReader>(
         let rendered_documentation =
             render_markdown(&documentation_content.clone(), MarkdownSource::Comment);
 
-        let functions: Vec<DocsFunction<'_>> = module
-            .ast
-            .definitions
-            .iter()
-            .filter(|statement| !statement.is_internal())
-            .flat_map(|statement| function(&source_links, statement))
-            .sorted()
-            .collect();
-
         let types: Vec<Type<'_>> = module
             .ast
             .definitions
@@ -187,12 +184,12 @@ pub fn generate_html<IO: FileSystemReader>(
             .sorted()
             .collect();
 
-        let constants: Vec<Constant<'_>> = module
+        let values: Vec<DocsValues<'_>> = module
             .ast
             .definitions
             .iter()
             .filter(|statement| !statement.is_internal())
-            .flat_map(|statement| constant(&source_links, statement))
+            .flat_map(|statement| value(&source_links, statement))
             .sorted()
             .collect();
 
@@ -228,9 +225,9 @@ pub fn generate_html<IO: FileSystemReader>(
                 reference: format!("{}.html#{}", module.name, type_.name),
             })
         });
-        constants.iter().for_each(|constant| {
+        values.iter().for_each(|constant| {
             search_items.push(SearchItem {
-                type_: SearchItemType::Constant,
+                type_: SearchItemType::Value,
                 parent_title: module.name.to_string(),
                 title: constant.name.to_string(),
                 content: format!(
@@ -242,20 +239,7 @@ pub fn generate_html<IO: FileSystemReader>(
                 reference: format!("{}.html#{}", module.name, constant.name),
             })
         });
-        functions.iter().for_each(|function| {
-            search_items.push(SearchItem {
-                type_: SearchItemType::Function,
-                parent_title: module.name.to_string(),
-                title: function.name.to_string(),
-                content: format!(
-                    "{}\n{}\n{}",
-                    function.signature,
-                    function.text_documentation,
-                    import_synonyms(&module.name, function.name)
-                ),
-                reference: format!("{}.html#{}", module.name, function.name),
-            })
-        });
+
         search_items.push(SearchItem {
             type_: SearchItemType::Module,
             parent_title: module.name.to_string(),
@@ -282,9 +266,8 @@ pub fn generate_html<IO: FileSystemReader>(
             module_name: EcoString::from(&name),
             file_path: &path.clone(),
             project_version: &config.version.to_string(),
-            functions,
             types,
-            constants,
+            values,
             rendering_timestamp: &rendering_timestamp,
         };
 
@@ -479,6 +462,20 @@ pub fn generate_json_package_interface(
     }
 }
 
+pub fn generate_json_package_information(path: Utf8PathBuf, config: PackageConfig) -> OutputFile {
+    OutputFile {
+        path,
+        content: Content::Text(package_information_as_json(config)),
+    }
+}
+
+fn package_information_as_json(config: PackageConfig) -> String {
+    let info = PackageInformation {
+        package_config: config,
+    };
+    serde_json::to_string_pretty(&info).expect("JSON module information serialisation")
+}
+
 fn page_unnest(path: &str) -> String {
     let unnest = path
         .strip_prefix('/')
@@ -541,48 +538,6 @@ fn import_synonyms(parent: &str, child: &str) -> String {
     format!("Synonyms:\n{parent}.{child}\n{parent} {child}")
 }
 
-fn function<'a>(
-    source_links: &SourceLinker,
-    statement: &'a TypedDefinition,
-) -> Option<DocsFunction<'a>> {
-    let mut formatter = format::Formatter::new();
-
-    match statement {
-        Definition::Function(Function {
-            publicity: Publicity::Public,
-            name,
-            documentation: doc,
-            arguments: args,
-            return_type: ret,
-            location,
-            deprecation,
-            ..
-        }) => {
-            let (_, name) = name
-                .as_ref()
-                .expect("Function in a definition must be named");
-
-            Some(DocsFunction {
-                name,
-                documentation: markdown_documentation(doc),
-                text_documentation: text_documentation(doc),
-                signature: print(
-                    formatter
-                        .docs_fn_signature(Publicity::Public, name, args, ret.clone(), location)
-                        .group(),
-                ),
-                source_url: source_links.url(*location),
-                deprecation_message: match deprecation {
-                    Deprecation::NotDeprecated => "".to_string(),
-                    Deprecation::Deprecated { message } => message.to_string(),
-                },
-            })
-        }
-
-        _ => None,
-    }
-}
-
 fn text_documentation(doc: &Option<(u32, EcoString)>) -> String {
     let raw_text = doc
         .as_ref()
@@ -723,12 +678,43 @@ fn type_<'a>(source_links: &SourceLinker, statement: &'a TypedDefinition) -> Opt
     }
 }
 
-fn constant<'a>(
+fn value<'a>(
     source_links: &SourceLinker,
     statement: &'a TypedDefinition,
-) -> Option<Constant<'a>> {
+) -> Option<DocsValues<'a>> {
     let mut formatter = format::Formatter::new();
     match statement {
+        Definition::Function(Function {
+            publicity: Publicity::Public,
+            name,
+            documentation: doc,
+            arguments: args,
+            return_type: ret,
+            location,
+            deprecation,
+            ..
+        }) => {
+            let (_, name) = name
+                .as_ref()
+                .expect("Function in a definition must be named");
+
+            Some(DocsValues {
+                name,
+                definition: print(
+                    formatter
+                        .docs_fn_signature(Publicity::Public, name, args, ret.clone(), location)
+                        .group(),
+                ),
+                documentation: markdown_documentation(doc),
+                text_documentation: text_documentation(doc),
+                source_url: source_links.url(*location),
+                deprecation_message: match deprecation {
+                    Deprecation::NotDeprecated => "".to_string(),
+                    Deprecation::Deprecated { message } => message.to_string(),
+                },
+            })
+        }
+
         Definition::ModuleConstant(ModuleConstant {
             publicity: Publicity::Public,
             documentation: doc,
@@ -736,12 +722,13 @@ fn constant<'a>(
             value,
             location,
             ..
-        }) => Some(Constant {
+        }) => Some(DocsValues {
             name,
             definition: print(formatter.docs_const_expr(Publicity::Public, name, value)),
             documentation: markdown_documentation(doc),
             text_documentation: text_documentation(doc),
             source_url: source_links.url(*location),
+            deprecation_message: "".to_string(),
         }),
 
         _ => None,
@@ -758,16 +745,6 @@ struct Link {
     path: String,
 }
 
-#[derive(PartialEq, Eq, PartialOrd, Ord)]
-struct DocsFunction<'a> {
-    name: &'a str,
-    signature: String,
-    documentation: String,
-    text_documentation: String,
-    source_url: String,
-    deprecation_message: String,
-}
-
 #[derive(PartialEq, Eq, PartialOrd, Ord, Debug)]
 struct TypeConstructor {
     definition: String,
@@ -795,12 +772,13 @@ struct Type<'a> {
 }
 
 #[derive(PartialEq, Eq, PartialOrd, Ord)]
-struct Constant<'a> {
+struct DocsValues<'a> {
     name: &'a str,
     definition: String,
     documentation: String,
     text_documentation: String,
     source_url: String,
+    deprecation_message: String,
 }
 
 #[derive(Template)]
@@ -836,9 +814,8 @@ struct ModuleTemplate<'a> {
     pages: &'a [Link],
     links: &'a [Link],
     modules: &'a [Link],
-    functions: Vec<DocsFunction<'a>>,
     types: Vec<Type<'a>>,
-    constants: Vec<Constant<'a>>,
+    values: Vec<DocsValues<'a>>,
     documentation: String,
     rendering_timestamp: &'a str,
 }
@@ -866,8 +843,7 @@ struct SearchItem {
 #[derive(Serialize, PartialEq, Eq, PartialOrd, Ord, Clone)]
 #[serde(rename_all = "lowercase")]
 enum SearchItemType {
-    Constant,
-    Function,
+    Value,
     Module,
     Page,
     Type,
@@ -880,3 +856,67 @@ enum SearchProgrammingLanguage {
     // Erlang,
     Gleam,
 }
+
+#[test]
+fn package_config_to_json() {
+    let input = r#"
+name = "my_project"
+version = "1.0.0"
+licences = ["Apache-2.0", "MIT"]
+description = "Pretty complex config"
+target = "erlang"
+repository = { type = "github", user = "example", repo = "my_dep" }
+links = [{ title = "Home page", href = "https://example.com" }]
+internal_modules = ["my_app/internal"]
+gleam = ">= 0.30.0"
+
+[dependencies]
+gleam_stdlib = ">= 0.18.0 and < 2.0.0"
+my_other_project = { path = "../my_other_project" }
+
+[dev-dependencies]
+gleeunit = ">= 1.0.0 and < 2.0.0"
+
+[documentation]
+pages = [{ title = "My Page", path = "my-page.html", source = "./path/to/my-page.md" }]
+
+[erlang]
+application_start_module = "my_app/application"
+extra_applications = ["inets", "ssl"]
+
+[javascript]
+typescript_declarations = true
+runtime = "node"
+
+[javascript.deno]
+allow_all = false
+allow_ffi = true
+allow_env = ["DATABASE_URL"]
+allow_net = ["example.com:443"]
+allow_read = ["./database.sqlite"]
+"#;
+
+    let config = toml::from_str::<PackageConfig>(&input).unwrap();
+    let info = PackageInformation {
+        package_config: config.clone(),
+    };
+    let json = package_information_as_json(config);
+    let output = format!("--- GLEAM.TOML\n{input}\n\n--- EXPORTED JSON\n\n{json}");
+    insta::assert_snapshot!(output);
+
+    let roundtrip: PackageInformation = serde_json::from_str(&json).unwrap();
+    assert_eq!(info, roundtrip);
+}
+
+#[test]
+fn barebones_package_config_to_json() {
+    let input = r#"
+name = "my_project"
+version = "1.0.0"
+"#;
+
+    let config = toml::from_str::<PackageConfig>(&input).unwrap();
+    let json = package_information_as_json(config);
+    let output = format!("--- GLEAM.TOML\n{input}\n\n--- EXPORTED JSON\n\n{json}");
+    insta::assert_snapshot!(output);
+}
