@@ -118,7 +118,7 @@ pub fn compile_test_project(src: &str, src_path: &str, dep: Option<(&str, &str,
     built_module.attach_doc_and_module_comments();
 
     let line_numbers = LineNumbers::new(src);
-    module(&built_module.ast, &line_numbers, &root)
+    module(&built_module.ast, &line_numbers, root)
         .unwrap()
         .replace(
             std::include_str!("../../templates/echo.erl"),
