@@ -1,4 +1,3 @@
-mod endianness;
 mod expression;
 mod import;
 mod pattern;
@@ -91,7 +90,7 @@ impl<'a> Generator<'a> {
             .name
             .as_str()
             .split('/')
-            .last()
+            .next_back()
             .expect("JavaScript generator could not identify imported module name.");
 
         docvec!["/// <reference types=\"./", module, ".d.mts\" />", line()]
@@ -454,7 +453,7 @@ impl<'a> Generator<'a> {
         let get_name = |module: &'a str| {
             module
                 .split('/')
-                .last()
+                .next_back()
                 .expect("JavaScript generator could not identify imported module name.")
         };
 
@@ -869,7 +868,7 @@ pub(crate) const SAFE_INT_SEGMENT_MAX_SIZE: usize = 48;
 pub(crate) fn bit_array_segment_int_value_to_bytes(
     mut value: BigInt,
     size: BigInt,
-    endianness: endianness::Endianness,
+    endianness: Endianness,
 ) -> Result<Vec<u8>, Error> {
     // Clamp negative sizes to zero
     let size = size.max(BigInt::ZERO);
