@@ -4,7 +4,6 @@ use std::sync::OnceLock;
 use super::{expression::is_js_scalar, *};
 use crate::{
     analyse::Inferred,
-    javascript::endianness::Endianness,
     strings::convert_string_escape_chars,
     type_::{FieldMap, PatternConstructor},
 };
@@ -33,7 +32,7 @@ enum Index<'a> {
 
 #[derive(Debug)]
 pub(crate) struct Generator<'module_ctx, 'expression_gen, 'a> {
-    pub expression_generator: &'expression_gen mut expression::Generator<'module_ctx>,
+    pub expression_generator: &'expression_gen mut expression::Generator<'module_ctx, 'a>,
     path: Vec<Index<'a>>,
     checks: Vec<Check<'a>>,
     assignments: Vec<Assignment<'a>>,
@@ -176,7 +175,7 @@ struct SizedBitArraySegmentDetails {
 
 impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, 'a> {
     pub fn new(
-        expression_generator: &'expression_gen mut expression::Generator<'module_ctx>,
+        expression_generator: &'expression_gen mut expression::Generator<'module_ctx, 'a>,
     ) -> Self {
         Self {
             path: vec![],
@@ -583,10 +582,10 @@ impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, '
                 Ok(())
             }
 
-            Pattern::Tuple { elems, .. } => {
+            Pattern::Tuple { elements, .. } => {
                 // We don't check the length because type system ensures it's a
                 // tuple of the correct size
-                for (index, pattern) in elems.iter().enumerate() {
+                for (index, pattern) in elements.iter().enumerate() {
                     self.push_int(index);
                     self.traverse_pattern(subject, pattern)?;
                     self.pop();
@@ -944,20 +943,34 @@ impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, '
             Endianness::Big
         };
 
+        let unit = segment
+            .options
+            .iter()
+            .find_map(|option| match option {
+                Opt::Unit { value, .. } => Some(*value),
+                _ => None,
+            })
+            .unwrap_or(1);
+
         let size = match segment
             .options
             .iter()
             .find(|x| matches!(x, Opt::Size { .. }))
         {
             Some(Opt::Size { value: size, .. }) => match &**size {
-                Pattern::Int { value, .. } => {
-                    Ok(BitArraySize::Literal(value.parse::<usize>().expect(
-                        "part of an Int node should always parse as integer",
-                    )))
-                }
-                Pattern::VarUsage { name, .. } => Ok(BitArraySize::Variable(
-                    self.expression_generator.local_var(name),
+                Pattern::Int { value, .. } => Ok(BitArraySize::Literal(
+                    value
+                        .parse::<usize>()
+                        .expect("part of an Int node should always parse as integer")
+                        * unit as usize,
                 )),
+                Pattern::VarUsage { name, .. } => {
+                    let mut variable = self.expression_generator.local_var(name);
+                    if unit != 1 {
+                        variable.push_str(&eco_format!(" * {unit}"));
+                    }
+                    Ok(BitArraySize::Variable(variable))
+                }
                 _ => Err(Error::Unsupported {
                     feature: "Non-constant size option in patterns".into(),
                     location: segment.location,
@@ -975,14 +988,6 @@ impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, '
             }
         }?;
 
-        // 16-bit floats are not supported
-        if segment.type_ == crate::type_::float() && size.is_constant_value(16) {
-            return Err(Error::Unsupported {
-                feature: "Float width of 16 bits in patterns".into(),
-                location: segment.location,
-            });
-        }
-
         let is_signed = segment
             .options
             .iter()
@@ -1246,7 +1251,7 @@ impl<'a> Check<'a> {
 }
 
 pub(crate) fn assign_subject<'a>(
-    expression_generator: &mut expression::Generator<'_>,
+    expression_generator: &mut expression::Generator<'_, 'a>,
     subject: &'a TypedExpr,
 ) -> (Document<'a>, Option<Document<'a>>) {
     static ASSIGNMENT_VAR_ECO_STR: OnceLock<EcoString> = OnceLock::new();
@@ -1272,7 +1277,7 @@ pub(crate) fn assign_subject<'a>(
 }
 
 pub(crate) fn assign_subjects<'a>(
-    expression_generator: &mut expression::Generator<'_>,
+    expression_generator: &mut expression::Generator<'_, 'a>,
     subjects: &'a [TypedExpr],
 ) -> Vec<(Document<'a>, Option<Document<'a>>)> {
     let mut out = Vec::with_capacity(subjects.len());
