@@ -37,10 +37,10 @@ mod use_;
 pub static CURRENT_PACKAGE: &str = "thepackage";
 
 #[macro_export]
-macro_rules! assert_js_with_multiple_imports {
-    ($(($name:literal, $module_src:literal)),+; $src:literal) => {
+macro_rules! assert_js {
+    ($(($name:literal, $module_src:literal)),+, $src:literal $(,)?) => {
         let compiled =
-            $crate::javascript::tests::compile_js($src, vec![$((CURRENT_PACKAGE, $name, $module_src)),*]).expect("compilation failed");
+            $crate::javascript::tests::compile_js($src, vec![$(($crate::javascript::tests::CURRENT_PACKAGE, $name, $module_src)),*]).expect("compilation failed");
             let mut output = String::from("----- SOURCE CODE\n");
             for (name, src) in [$(($name, $module_src)),*] {
                 output.push_str(&format!("-- {name}.gleam\n{src}\n\n"));
@@ -48,10 +48,7 @@ macro_rules! assert_js_with_multiple_imports {
             output.push_str(&format!("-- main.gleam\n{}\n\n----- COMPILED JAVASCRIPT\n{compiled}", $src));
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
-}
 
-#[macro_export]
-macro_rules! assert_js {
     (($dep_package:expr, $dep_name:expr, $dep_src:expr), $src:expr $(,)?) => {{
         let compiled =
             $crate::javascript::tests::compile_js($src, vec![($dep_package, $dep_name, $dep_src)])
@@ -87,15 +84,6 @@ macro_rules! assert_js {
     }};
 }
 
-#[macro_export]
-macro_rules! assert_js_error {
-    ($src:expr $(,)?) => {{
-        let error = $crate::javascript::tests::expect_js_error($src, vec![]);
-        let output = format!("----- SOURCE CODE\n{}\n\n----- ERROR\n{}", $src, error);
-        insta::assert_snapshot!(insta::internals::AutoName, output, $src);
-    }};
-}
-
 #[macro_export]
 macro_rules! assert_ts_def {
     (($dep_1_package:expr, $dep_1_name:expr, $dep_1_src:expr), ($dep_2_package:expr, $dep_2_name:expr, $dep_2_src:expr), $src:expr $(,)?) => {{
@@ -226,19 +214,3 @@ pub fn compile_ts(src: &str, deps: Vec<(&str, &str, &str)>) -> Result<String, cr
     let ast = compile(src, deps);
     ts_declaration(&ast, Utf8Path::new(""), &src.into())
 }
-
-pub fn expect_js_error(src: &str, deps: Vec<(&str, &str, &str)>) -> String {
-    let error = compile_js(src, deps).expect_err("should not compile");
-    println!("er: {error:#?}");
-    let better_error = match error {
-        crate::Error::JavaScript {
-            error: inner_error, ..
-        } => crate::Error::JavaScript {
-            src: src.into(),
-            path: Utf8PathBuf::from("/src/javascript/error.gleam"),
-            error: inner_error,
-        },
-        _ => panic!("expected js error, got {error:#?}"),
-    };
-    better_error.pretty_string()
-}
