@@ -2,8 +2,8 @@ use hexpm::version::Version;
 use pubgrub::range::Range;
 
 use crate::{
-    assert_js, assert_js_error, assert_js_no_warnings_with_gleam_version,
-    assert_js_warnings_with_gleam_version, assert_ts_def,
+    assert_js, assert_js_no_warnings_with_gleam_version, assert_js_warnings_with_gleam_version,
+    assert_ts_def,
 };
 
 #[test]
@@ -787,8 +787,8 @@ fn go(x) {
 }
 
 #[test]
-fn match_float_16_bit_error() {
-    assert_js_error!(
+fn match_float_16_bit() {
+    assert_js!(
         r#"
 fn go(x) {
   let assert <<a:float-size(16)>> = x
@@ -1048,3 +1048,49 @@ pub fn main() {
 }"#
     );
 }
+
+#[test]
+fn with_unit() {
+    assert_js!(
+        r#"
+fn main() {
+  <<1:size(2)-unit(2), 2:size(3)-unit(4)>>
+}
+"#,
+    );
+}
+
+#[test]
+fn dynamic_size_with_unit() {
+    assert_js!(
+        r#"
+fn main() {
+  let size = 3
+  <<1:size(size)-unit(2)>>
+}
+"#,
+    );
+}
+
+#[test]
+fn pattern_with_unit() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1:size(2)-unit(2), 2:size(3)-unit(4)>> = x
+}
+"#,
+    );
+}
+
+#[test]
+fn dynamic_size_pattern_with_unit() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let size = 3
+  let assert <<1:size(size)-unit(2)>> = x
+}
+"#,
+    );
+}
