@@ -402,22 +402,10 @@ where
 
         let (insert_range, module_select) = surrounding_completion;
 
-        // Prelude types
-        for type_ in PreludeType::iter() {
-            let label: String = type_.name().into();
-            let sort_text = Some(sort_text(CompletionKind::Prelude, &label));
-            completions.push(CompletionItem {
-                label,
-                detail: Some("Type".into()),
-                kind: Some(CompletionItemKind::CLASS),
-                sort_text,
-                ..Default::default()
-            });
-        }
-
-        // Module types
+        // Module and prelude types
         // Do not complete direct module types if the user has already started typing a module select.
-        // e.x. when the user has typed mymodule.| we know local module types are no longer relevant
+        // e.x. when the user has typed mymodule.| we know local module types and prelude types are no
+        // longer relevant.
         if module_select.is_none() {
             for (name, type_) in &self.module.ast.type_info.types {
                 completions.push(type_completion(
@@ -429,6 +417,18 @@ where
                     CompletionKind::LocallyDefined,
                 ));
             }
+
+            for type_ in PreludeType::iter() {
+                let label: String = type_.name().into();
+                let sort_text = Some(sort_text(CompletionKind::Prelude, &label));
+                completions.push(CompletionItem {
+                    label,
+                    detail: Some("Type".into()),
+                    kind: Some(CompletionItemKind::CLASS),
+                    sort_text,
+                    ..Default::default()
+                });
+            }
         }
 
         // Imported modules
@@ -469,16 +469,15 @@ where
             // e.x. when the user has typed mymodule.| we know unqualified module types are no longer relevant.
             if module_select.is_none() {
                 for unqualified in &import.unqualified_types {
-                    match module.get_public_type(&unqualified.name) {
-                        Some(type_) => completions.push(type_completion(
+                    if let Some(type_) = module.get_public_type(&unqualified.name) {
+                        completions.push(type_completion(
                             None,
                             unqualified.used_name(),
                             type_,
                             insert_range,
                             TypeCompletionForm::Default,
                             CompletionKind::ImportedModule,
-                        )),
-                        None => continue,
+                        ))
                     }
                 }
             }
@@ -504,7 +503,7 @@ where
 
             let qualifier = module_full_name
                 .split('/')
-                .last()
+                .next_back()
                 .unwrap_or(module_full_name);
 
             // If the user has already started a module select then don't show irrelevant modules.
@@ -550,44 +549,10 @@ where
 
         let (insert_range, module_select) = surrounding_completion;
 
-        let mut push_prelude_completion = |label: &str, kind| {
-            let label = label.to_string();
-            let sort_text = Some(sort_text(CompletionKind::Prelude, &label));
-            completions.push(CompletionItem {
-                label,
-                detail: Some(PRELUDE_MODULE_NAME.into()),
-                kind: Some(kind),
-                sort_text,
-                ..Default::default()
-            });
-        };
-
-        // Prelude values
-        for type_ in PreludeType::iter() {
-            match type_ {
-                PreludeType::Bool => {
-                    push_prelude_completion("True", CompletionItemKind::ENUM_MEMBER);
-                    push_prelude_completion("False", CompletionItemKind::ENUM_MEMBER);
-                }
-                PreludeType::Nil => {
-                    push_prelude_completion("Nil", CompletionItemKind::ENUM_MEMBER);
-                }
-                PreludeType::Result => {
-                    push_prelude_completion("Ok", CompletionItemKind::CONSTRUCTOR);
-                    push_prelude_completion("Error", CompletionItemKind::CONSTRUCTOR);
-                }
-                PreludeType::BitArray
-                | PreludeType::Float
-                | PreludeType::Int
-                | PreludeType::List
-                | PreludeType::String
-                | PreludeType::UtfCodepoint => {}
-            }
-        }
-
-        // Module values
+        // Module and prelude values
         // Do not complete direct module values if the user has already started typing a module select.
-        // e.x. when the user has typed mymodule.| we know local module values are no longer relevant
+        // e.x. when the user has typed mymodule.| we know local module and prelude values are no longer
+        // relevant.
         if module_select.is_none() {
             let cursor = self
                 .src_line_numbers
@@ -617,6 +582,40 @@ where
                     CompletionKind::LocallyDefined,
                 ));
             }
+
+            let mut push_prelude_completion = |label: &str, kind| {
+                let label = label.to_string();
+                let sort_text = Some(sort_text(CompletionKind::Prelude, &label));
+                completions.push(CompletionItem {
+                    label,
+                    detail: Some(PRELUDE_MODULE_NAME.into()),
+                    kind: Some(kind),
+                    sort_text,
+                    ..Default::default()
+                });
+            };
+
+            for type_ in PreludeType::iter() {
+                match type_ {
+                    PreludeType::Bool => {
+                        push_prelude_completion("True", CompletionItemKind::ENUM_MEMBER);
+                        push_prelude_completion("False", CompletionItemKind::ENUM_MEMBER);
+                    }
+                    PreludeType::Nil => {
+                        push_prelude_completion("Nil", CompletionItemKind::ENUM_MEMBER);
+                    }
+                    PreludeType::Result => {
+                        push_prelude_completion("Ok", CompletionItemKind::CONSTRUCTOR);
+                        push_prelude_completion("Error", CompletionItemKind::CONSTRUCTOR);
+                    }
+                    PreludeType::BitArray
+                    | PreludeType::Float
+                    | PreludeType::Int
+                    | PreludeType::List
+                    | PreludeType::String
+                    | PreludeType::UtfCodepoint => {}
+                }
+            }
         }
 
         // Imported modules
@@ -657,19 +656,16 @@ where
             // e.x. when the user has typed mymodule.| we know unqualified module values are no longer relevant.
             if module_select.is_none() {
                 for unqualified in &import.unqualified_values {
-                    match module.get_public_value(&unqualified.name) {
-                        Some(value) => {
-                            let name = unqualified.used_name();
-                            completions.push(value_completion(
-                                None,
-                                mod_name,
-                                name,
-                                value,
-                                insert_range,
-                                CompletionKind::ImportedModule,
-                            ))
-                        }
-                        None => continue,
+                    if let Some(value) = module.get_public_value(&unqualified.name) {
+                        let name = unqualified.used_name();
+                        completions.push(value_completion(
+                            None,
+                            mod_name,
+                            name,
+                            value,
+                            insert_range,
+                            CompletionKind::ImportedModule,
+                        ))
                     }
                 }
             }
@@ -693,7 +689,7 @@ where
             }
             let qualifier = module_full_name
                 .split('/')
-                .last()
+                .next_back()
                 .unwrap_or(module_full_name);
 
             // If the user has already started a module select then don't show irrelevant modules.
