@@ -2,7 +2,7 @@ use crate::{
     Error, Result, Warning,
     analyse::name::correct_name_case,
     ast::{
-        self, ArgNames, CustomType, Definition, DefinitionLocation, ModuleConstant, Pattern,
+        self, CustomType, Definition, DefinitionLocation, ModuleConstant, PatternUnusedArguments,
         SrcSpan, TypedArg, TypedExpr, TypedFunction, TypedModule, TypedPattern,
     },
     build::{Located, Module, UnqualifiedImport, type_constructor_from_modules},
@@ -15,7 +15,9 @@ use crate::{
     paths::ProjectPaths,
     type_::{
         self, Deprecation, ModuleInterface, Type, TypeConstructor, ValueConstructor,
-        ValueConstructorVariant, error::VariableOrigin, printer::Printer,
+        ValueConstructorVariant,
+        error::{Named, VariableOrigin},
+        printer::Printer,
     },
 };
 use camino::Utf8PathBuf;
@@ -33,16 +35,22 @@ use super::{
     DownloadDependencies, MakeLocker,
     code_action::{
         AddAnnotations, CodeActionBuilder, ConvertFromUse, ConvertToFunctionCall, ConvertToPipe,
-        ConvertToUse, ExpandFunctionCapture, ExtractVariable, FillInMissingLabelledArgs,
-        GenerateDynamicDecoder, GenerateFunction, GenerateJsonEncoder, InlineVariable,
-        InterpolateString, LetAssertToCase, PatternMatchOnValue, RedundantTupleInCaseSubject,
-        UseLabelShorthandSyntax, code_action_add_missing_patterns,
+        ConvertToUse, ExpandFunctionCapture, ExtractConstant, ExtractVariable,
+        FillInMissingLabelledArgs, FillUnusedFields, FixBinaryOperation, GenerateDynamicDecoder,
+        GenerateFunction, GenerateJsonEncoder, InlineVariable, InterpolateString, LetAssertToCase,
+        PatternMatchOnValue, RedundantTupleInCaseSubject, RemoveEchos, UseLabelShorthandSyntax,
+        WrapInBlock, code_action_add_missing_patterns,
         code_action_convert_qualified_constructor_to_unqualified,
         code_action_convert_unqualified_constructor_to_qualified, code_action_import_module,
         code_action_inexhaustive_let_to_case,
     },
     completer::Completer,
-    rename::{VariableRenameKind, rename_local_variable},
+    reference::{
+        Referenced, find_module_references, find_variable_references, reference_for_ast_node,
+    },
+    rename::{
+        RenameTarget, Renamed, VariableRenameKind, rename_local_variable, rename_module_entity,
+    },
     signature_help, src_span_to_lsp_range,
 };
 
@@ -300,9 +308,8 @@ where
 
                 Located::FunctionBody(_) => Some(completer.completion_values()),
 
-                Located::ModuleStatement(Definition::TypeAlias(_) | Definition::CustomType(_)) => {
-                    Some(completer.completion_types())
-                }
+                Located::ModuleStatement(Definition::TypeAlias(_) | Definition::CustomType(_))
+                | Located::VariantConstructorDefinition(_) => Some(completer.completion_types()),
 
                 // If the import completions returned no results and we are in an import then
                 // we should try to provide completions for unqualified values
@@ -319,7 +326,7 @@ where
 
                 Located::Arg(_) => None,
 
-                Located::Annotation(_, _) => Some(completer.completion_types()),
+                Located::Annotation { .. } => Some(completer.completion_types()),
 
                 Located::Label(_, _) => None,
 
@@ -373,16 +380,20 @@ where
                 &this.error,
                 &mut actions,
             );
+            actions.extend(FixBinaryOperation::new(module, &lines, &params).code_actions());
             actions.extend(LetAssertToCase::new(module, &lines, &params).code_actions());
             actions
                 .extend(RedundantTupleInCaseSubject::new(module, &lines, &params).code_actions());
             actions.extend(UseLabelShorthandSyntax::new(module, &lines, &params).code_actions());
             actions.extend(FillInMissingLabelledArgs::new(module, &lines, &params).code_actions());
             actions.extend(ConvertFromUse::new(module, &lines, &params).code_actions());
+            actions.extend(RemoveEchos::new(module, &lines, &params).code_actions());
             actions.extend(ConvertToUse::new(module, &lines, &params).code_actions());
             actions.extend(ExpandFunctionCapture::new(module, &lines, &params).code_actions());
+            actions.extend(FillUnusedFields::new(module, &lines, &params).code_actions());
             actions.extend(InterpolateString::new(module, &lines, &params).code_actions());
             actions.extend(ExtractVariable::new(module, &lines, &params).code_actions());
+            actions.extend(ExtractConstant::new(module, &lines, &params).code_actions());
             actions.extend(GenerateFunction::new(module, &lines, &params).code_actions());
             actions.extend(ConvertToPipe::new(module, &lines, &params).code_actions());
             actions.extend(ConvertToFunctionCall::new(module, &lines, &params).code_actions());
@@ -390,8 +401,16 @@ where
                 PatternMatchOnValue::new(module, &lines, &params, &this.compiler).code_actions(),
             );
             actions.extend(InlineVariable::new(module, &lines, &params).code_actions());
+            actions.extend(WrapInBlock::new(module, &lines, &params).code_actions());
             GenerateDynamicDecoder::new(module, &lines, &params, &mut actions).code_actions();
-            GenerateJsonEncoder::new(module, &lines, &params, &mut actions).code_actions();
+            GenerateJsonEncoder::new(
+                module,
+                &lines,
+                &params,
+                &mut actions,
+                &this.compiler.project_compiler.config,
+            )
+            .code_actions();
             AddAnnotations::new(module, &lines, &params).code_action(&mut actions);
             Ok(if actions.is_empty() {
                 None
@@ -546,6 +565,20 @@ where
         })
     }
 
+    /// Check whether a particular module is in the same package as this one
+    fn is_same_package(&self, current_module: &Module, module_name: &str) -> bool {
+        let other_module = self
+            .compiler
+            .project_compiler
+            .get_importable_modules()
+            .get(module_name);
+        match other_module {
+            // We can't rename values from other packages if we are not aliasing an unqualified import.
+            Some(module) => module.package == current_module.ast.type_info.package,
+            None => false,
+        }
+    }
+
     pub fn prepare_rename(
         &mut self,
         params: lsp::TextDocumentPositionParams,
@@ -556,50 +589,55 @@ where
                 None => return Ok(None),
             };
 
+            let Some(current_module) = this.module_for_uri(&params.text_document.uri) else {
+                return Ok(None);
+            };
+
             let success_response = |location| {
                 Some(PrepareRenameResponse::Range(src_span_to_lsp_range(
                     location, &lines,
                 )))
             };
 
-            Ok(match found {
-                Located::Expression(TypedExpr::Var {
-                    constructor:
-                        ValueConstructor {
-                            variant: ValueConstructorVariant::LocalVariable { origin, location },
-                            ..
-                        },
-                    ..
-                })
-                | Located::Pattern(Pattern::Variable {
-                    origin, location, ..
-                }) => match origin {
-                    VariableOrigin::Variable(_)
-                    | VariableOrigin::AssignmentPattern
-                    | VariableOrigin::LabelShorthand(_) => success_response(*location),
-                    VariableOrigin::Generated => None,
+            let byte_index = lines.byte_index(params.position.line, params.position.character);
+
+            Ok(match reference_for_ast_node(found, &current_module.name) {
+                Some(Referenced::LocalVariable {
+                    location, origin, ..
+                }) if location.contains(byte_index) => match origin {
+                    Some(VariableOrigin::Generated) => None,
+                    Some(
+                        VariableOrigin::Variable(_)
+                        | VariableOrigin::AssignmentPattern
+                        | VariableOrigin::LabelShorthand(_),
+                    )
+                    | None => success_response(location),
                 },
-                Located::Pattern(Pattern::VarUsage { constructor, .. }) => constructor
-                    .as_ref()
-                    .and_then(|constructor| match &constructor.variant {
-                        ValueConstructorVariant::LocalVariable { origin, location } => match origin
-                        {
-                            VariableOrigin::Variable(_)
-                            | VariableOrigin::AssignmentPattern
-                            | VariableOrigin::LabelShorthand(_) => success_response(*location),
-                            VariableOrigin::Generated => None,
-                        },
-                        _ => None,
-                    }),
-                Located::Pattern(Pattern::Assign { location, .. }) => success_response(*location),
-                Located::Arg(arg) => match &arg.names {
-                    ArgNames::Named { location, .. }
-                    | ArgNames::NamedLabelled {
-                        name_location: location,
+                Some(
+                    Referenced::ModuleValue {
+                        module,
+                        location,
+                        target_kind,
                         ..
-                    } => success_response(*location),
-                    ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => None,
-                },
+                    }
+                    | Referenced::ModuleType {
+                        module,
+                        location,
+                        target_kind,
+                        ..
+                    },
+                ) if location.contains(byte_index) => {
+                    // We can't rename types or values from other packages if we are not aliasing an unqualified import.
+                    let rename_allowed = match target_kind {
+                        RenameTarget::Qualified => this.is_same_package(current_module, &module),
+                        RenameTarget::Unqualified | RenameTarget::Definition => true,
+                    };
+                    if rename_allowed {
+                        success_response(location)
+                    } else {
+                        None
+                    }
+                }
                 _ => None,
             })
         })
@@ -618,82 +656,131 @@ where
                 return Ok(None);
             };
 
-            Ok(match found {
-                Located::Expression(TypedExpr::Var {
-                    constructor:
-                        ValueConstructor {
-                            variant: ValueConstructorVariant::LocalVariable { location, origin },
-                            ..
-                        },
+            Ok(match reference_for_ast_node(found, &module.name) {
+                Some(Referenced::LocalVariable {
+                    origin,
+                    definition_location,
                     ..
-                })
-                | Located::Pattern(Pattern::Variable {
-                    location, origin, ..
-                }) => match origin {
-                    VariableOrigin::Variable(_) | VariableOrigin::AssignmentPattern => {
-                        rename_local_variable(
-                            module,
-                            &lines,
-                            &params,
-                            *location,
-                            VariableRenameKind::Variable,
-                        )
-                    }
-                    VariableOrigin::LabelShorthand(_) => rename_local_variable(
-                        module,
-                        &lines,
-                        &params,
-                        *location,
-                        VariableRenameKind::LabelShorthand,
-                    ),
-                    VariableOrigin::Generated => None,
-                },
-                Located::Pattern(Pattern::VarUsage { constructor, .. }) => constructor
-                    .as_ref()
-                    .and_then(|constructor| match &constructor.variant {
-                        ValueConstructorVariant::LocalVariable { location, origin } => match origin
-                        {
-                            VariableOrigin::Variable(_) | VariableOrigin::AssignmentPattern => {
-                                rename_local_variable(
-                                    module,
-                                    &lines,
-                                    &params,
-                                    *location,
-                                    VariableRenameKind::Variable,
-                                )
-                            }
-                            VariableOrigin::LabelShorthand(_) => rename_local_variable(
-                                module,
-                                &lines,
-                                &params,
-                                *location,
-                                VariableRenameKind::LabelShorthand,
-                            ),
-                            VariableOrigin::Generated => None,
-                        },
-                        _ => None,
-                    }),
-                Located::Pattern(Pattern::Assign { location, .. }) => rename_local_variable(
-                    module,
-                    &lines,
+                }) => {
+                    let rename_kind = match origin {
+                        Some(VariableOrigin::Generated) => return Ok(None),
+                        Some(VariableOrigin::LabelShorthand(_)) => {
+                            VariableRenameKind::LabelShorthand
+                        }
+                        Some(VariableOrigin::AssignmentPattern | VariableOrigin::Variable(_))
+                        | None => VariableRenameKind::Variable,
+                    };
+                    rename_local_variable(module, &lines, &params, definition_location, rename_kind)
+                }
+                Some(Referenced::ModuleValue {
+                    module: module_name,
+                    target_kind,
+                    name,
+                    name_kind,
+                    ..
+                }) => rename_module_entity(
                     &params,
-                    *location,
-                    VariableRenameKind::Variable,
+                    module,
+                    this.compiler.project_compiler.get_importable_modules(),
+                    &this.compiler.sources,
+                    Renamed {
+                        module_name: &module_name,
+                        name: &name,
+                        name_kind,
+                        target_kind,
+                        layer: ast::Layer::Value,
+                    },
                 ),
-                Located::Arg(arg) => match &arg.names {
-                    ArgNames::Named { location, .. }
-                    | ArgNames::NamedLabelled {
-                        name_location: location,
-                        ..
-                    } => rename_local_variable(
-                        module,
-                        &lines,
-                        &params,
-                        *location,
-                        VariableRenameKind::Variable,
+                Some(Referenced::ModuleType {
+                    module: module_name,
+                    target_kind,
+                    name,
+                    ..
+                }) => rename_module_entity(
+                    &params,
+                    module,
+                    this.compiler.project_compiler.get_importable_modules(),
+                    &this.compiler.sources,
+                    Renamed {
+                        module_name: &module_name,
+                        name: &name,
+                        name_kind: Named::Type,
+                        target_kind,
+                        layer: ast::Layer::Type,
+                    },
+                ),
+                None => None,
+            })
+        })
+    }
+
+    pub fn find_references(
+        &mut self,
+        params: lsp::ReferenceParams,
+    ) -> Response<Option<Vec<lsp::Location>>> {
+        self.respond(|this| {
+            let position = &params.text_document_position;
+
+            let (lines, found) = match this.node_at_position(position) {
+                Some(value) => value,
+                None => return Ok(None),
+            };
+
+            let uri = position.text_document.uri.clone();
+
+            let Some(module) = this.module_for_uri(&uri) else {
+                return Ok(None);
+            };
+
+            let byte_index = lines.byte_index(position.position.line, position.position.character);
+
+            Ok(match reference_for_ast_node(found, &module.name) {
+                Some(Referenced::LocalVariable {
+                    origin,
+                    definition_location,
+                    location,
+                }) if location.contains(byte_index) => match origin {
+                    Some(VariableOrigin::Generated) => None,
+                    Some(
+                        VariableOrigin::LabelShorthand(_)
+                        | VariableOrigin::AssignmentPattern
+                        | VariableOrigin::Variable(_),
+                    )
+                    | None => Some(
+                        find_variable_references(&module.ast, definition_location)
+                            .into_iter()
+                            .chain(std::iter::once(definition_location))
+                            .map(|location| lsp::Location {
+                                uri: uri.clone(),
+                                range: src_span_to_lsp_range(location, &lines),
+                            })
+                            .collect(),
                     ),
-                    ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => None,
                 },
+                Some(Referenced::ModuleValue {
+                    module,
+                    name,
+                    location,
+                    ..
+                }) if location.contains(byte_index) => Some(find_module_references(
+                    module,
+                    name,
+                    this.compiler.project_compiler.get_importable_modules(),
+                    &this.compiler.sources,
+                    ast::Layer::Value,
+                )),
+                Some(Referenced::ModuleType {
+                    module,
+                    name,
+                    location,
+                    ..
+                }) if location.contains(byte_index) => Some(find_module_references(
+                    module,
+                    name,
+                    this.compiler.project_compiler.get_importable_modules(),
+                    &this.compiler.sources,
+                    ast::Layer::Type,
+                )),
                 _ => None,
             })
         })
@@ -750,6 +837,7 @@ where
                     ))
                 }
                 Located::ModuleStatement(_) => None,
+                Located::VariantConstructorDefinition(_) => None,
                 Located::UnqualifiedImport(UnqualifiedImport {
                     name,
                     module: module_name,
@@ -783,30 +871,28 @@ where
                 Located::Pattern(pattern) => Some(hover_for_pattern(pattern, lines, module)),
                 Located::PatternSpread {
                     spread_location,
-                    arguments,
+                    pattern,
                 } => {
                     let range = Some(src_span_to_lsp_range(spread_location, &lines));
 
-                    let mut positional = vec![];
-                    let mut labelled = vec![];
-                    for argument in arguments {
-                        // We only want to display the arguments that were ignored using `..`.
-                        // Any argument ignored that way is marked as implicit, so if it is
-                        // not implicit we just ignore it.
-                        if !argument.is_implicit() {
-                            continue;
-                        }
+                    let mut printer = Printer::new(&module.ast.names);
 
-                        let type_ = Printer::new(&module.ast.names)
-                            .print_type(argument.value.type_().as_ref());
-                        match &argument.label {
-                            Some(label) => labelled.push(format!("- `{label}: {type_}`")),
-                            None => positional.push(format!("- `{type_}`")),
-                        }
-                    }
+                    let PatternUnusedArguments {
+                        positional,
+                        labelled,
+                    } = pattern.unused_arguments().unwrap_or_default();
+
+                    let positional = positional
+                        .iter()
+                        .map(|type_| format!("- `{}`", printer.print_type(type_)))
+                        .join("\n");
+                    let labelled = labelled
+                        .iter()
+                        .map(|(label, type_)| {
+                            format!("- `{}: {}`", label, printer.print_type(type_))
+                        })
+                        .join("\n");
 
-                    let positional = positional.join("\n");
-                    let labelled = labelled.join("\n");
                     let content = match (positional.is_empty(), labelled.is_empty()) {
                         (true, false) => format!("Unused labelled fields:\n{labelled}"),
                         (false, true) => format!("Unused positional fields:\n{positional}"),
@@ -832,13 +918,13 @@ Unused labelled fields:
                 )),
                 Located::Arg(arg) => Some(hover_for_function_argument(arg, lines, module)),
                 Located::FunctionBody(_) => None,
-                Located::Annotation(annotation, type_) => {
+                Located::Annotation { ast, type_ } => {
                     let type_constructor = type_constructor_from_modules(
                         this.compiler.project_compiler.get_importable_modules(),
                         type_.clone(),
                     );
                     Some(hover_for_annotation(
-                        annotation,
+                        ast.location(),
                         &type_,
                         type_constructor,
                         lines,
@@ -1053,7 +1139,7 @@ fn hover_for_pattern(pattern: &TypedPattern, line_numbers: LineNumbers, module:
 fn get_function_type(fun: &TypedFunction) -> Type {
     Type::Fn {
         args: fun.arguments.iter().map(|arg| arg.type_.clone()).collect(),
-        retrn: fun.return_type.clone(),
+        return_: fun.return_type.clone(),
     }
 }
 
