@@ -4,7 +4,9 @@ mod completion;
 mod definition;
 mod document_symbols;
 mod hover;
+mod reference;
 mod rename;
+mod router;
 mod signature_help;
 
 use std::{
@@ -260,9 +262,9 @@ impl TestLocker {
 }
 
 impl Locker for TestLocker {
-    fn lock_for_build(&self) -> LockGuard {
+    fn lock_for_build(&self) -> Result<LockGuard> {
         self.record(Action::LockBuild);
-        LockGuard(Box::new(Guard(self.actions.clone())))
+        Ok(LockGuard(Box::new(Guard(self.actions.clone()))))
     }
 }
 
@@ -396,6 +398,7 @@ struct TestProject<'a> {
     hex_modules: Vec<(&'a str, &'a str)>,
     dev_hex_modules: Vec<(&'a str, &'a str)>,
     indirect_hex_modules: Vec<(&'a str, &'a str)>,
+    package_modules: HashMap<&'a str, Vec<(&'a str, &'a str)>>,
 }
 
 impl<'a> TestProject<'a> {
@@ -408,17 +411,23 @@ impl<'a> TestProject<'a> {
             hex_modules: vec![],
             dev_hex_modules: vec![],
             indirect_hex_modules: vec![],
+            package_modules: HashMap::new(),
         }
     }
 
+    pub fn module_name_from_url(&self, url: &Url) -> Option<String> {
+        Some(
+            url.path_segments()?
+                .skip_while(|segment| *segment != "src")
+                .skip(1)
+                .join("/")
+                .trim_end_matches(".gleam")
+                .into(),
+        )
+    }
+
     pub fn src_from_module_url(&self, url: &Url) -> Option<&str> {
-        let module_name: EcoString = url
-            .path_segments()?
-            .skip_while(|segment| *segment != "src")
-            .skip(1)
-            .join("/")
-            .trim_end_matches(".gleam")
-            .into();
+        let module_name: EcoString = self.module_name_from_url(url)?.into();
 
         if module_name == "app" {
             return Some(self.src);
@@ -469,6 +478,14 @@ impl<'a> TestProject<'a> {
         self
     }
 
+    pub fn add_package_module(mut self, package: &'a str, name: &'a str, src: &'a str) -> Self {
+        self.package_modules
+            .entry(package)
+            .or_default()
+            .push((name, src));
+        self
+    }
+
     pub fn build_engine(
         &self,
         io: &mut LanguageServerTestIO,
@@ -486,6 +503,13 @@ impl<'a> TestProject<'a> {
             _ = io.hex_dep_module("indirect_hex", name, code);
         });
 
+        for (package, modules) in self.package_modules.iter() {
+            io.add_hex_package(package);
+            for (module, code) in modules {
+                _ = io.hex_dep_module(package, module, code);
+            }
+        }
+
         let mut engine = setup_engine(io);
 
         // Add an external dependency and all its modules
