@@ -12,16 +12,17 @@ use crate::{
     },
     build::Origin,
     line_numbers::LineNumbers,
+    reference::{Reference, ReferenceKind, ReferenceMap},
     schema_capnp::{self as schema, *},
     type_::{
-        self, AccessorsMap, Deprecation, FieldMap, ModuleInterface, Opaque, RecordAccessor, Type,
-        TypeAliasConstructor, TypeConstructor, TypeValueConstructor, TypeValueConstructorField,
-        TypeVariantConstructors, ValueConstructor, ValueConstructorVariant,
-        expression::Implementations,
+        self, AccessorsMap, Deprecation, FieldMap, ModuleInterface, Opaque, RecordAccessor,
+        References, Type, TypeAliasConstructor, TypeConstructor, TypeValueConstructor,
+        TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
+        ValueConstructorVariant, expression::Implementations,
     },
     uid::UniqueIdGenerator,
 };
-use std::{collections::HashMap, io::BufRead, sync::Arc};
+use std::{collections::HashMap, collections::HashSet, io::BufRead, sync::Arc};
 
 macro_rules! read_vec {
     ($reader:expr, $self:expr, $method:ident) => {{
@@ -87,6 +88,7 @@ impl ModuleDecoder {
             type_aliases: read_hashmap!(reader.get_type_aliases()?, self, type_alias_constructor),
             documentation: self.string_list(reader.get_documentation()?)?,
             contains_echo: reader.get_contains_echo(),
+            references: self.references(reader.get_references()?)?,
         })
     }
 
@@ -108,6 +110,54 @@ impl ModuleDecoder {
             .map_err(|_| capnp::Error::failed("String contains non-utf8 characters".into()).into())
     }
 
+    fn references(&self, reader: references::Reader<'_>) -> Result<References> {
+        Ok(References {
+            imported_modules: self.string_set(reader.get_imported_modules()?)?,
+            value_references: self.reference_map(reader.get_value_references()?)?,
+            type_references: self.reference_map(reader.get_type_references()?)?,
+        })
+    }
+
+    fn string_set(&self, reader: text_list::Reader<'_>) -> Result<HashSet<EcoString>> {
+        let mut set = HashSet::with_capacity(reader.len() as usize);
+        for reader in reader.into_iter() {
+            let _ = set.insert(self.string(reader?)?);
+        }
+        Ok(set)
+    }
+
+    fn reference_map(
+        &self,
+        reader: capnp::struct_list::Reader<'_, reference_map::Owned>,
+    ) -> Result<ReferenceMap> {
+        let mut map = HashMap::with_capacity(reader.len() as usize);
+        for prop in reader.into_iter() {
+            let module = self.string(prop.get_module()?)?;
+            let name = self.string(prop.get_name()?)?;
+            let references = read_vec!(prop.get_references()?, self, reference);
+            let _ = map.insert((module, name), references);
+        }
+        Ok(map)
+    }
+
+    fn reference(&self, reader: &reference::Reader<'_>) -> Result<Reference> {
+        Ok(Reference {
+            location: self.src_span(&reader.get_location()?)?,
+            kind: self.reference_kind(&reader.get_kind()?)?,
+        })
+    }
+
+    fn reference_kind(&self, reader: &reference_kind::Reader<'_>) -> Result<ReferenceKind> {
+        use reference_kind::Which;
+        Ok(match reader.which()? {
+            Which::Qualified(_) => ReferenceKind::Qualified,
+            Which::Unqualified(_) => ReferenceKind::Unqualified,
+            Which::Import(_) => ReferenceKind::Import,
+            Which::Definition(_) => ReferenceKind::Definition,
+            Which::Alias(_) => ReferenceKind::Alias,
+        })
+    }
+
     fn type_constructor(
         &mut self,
         reader: &type_constructor::Reader<'_>,
@@ -184,14 +234,14 @@ impl ModuleDecoder {
     }
 
     fn type_fn(&mut self, reader: &schema::type_::fn_::Reader<'_>) -> Result<Arc<Type>> {
-        let retrn = self.type_(&reader.get_return()?)?;
+        let return_ = self.type_(&reader.get_return()?)?;
         let args = read_vec!(&reader.get_arguments()?, self, type_);
-        Ok(Arc::new(Type::Fn { args, retrn }))
+        Ok(Arc::new(Type::Fn { args, return_ }))
     }
 
     fn type_tuple(&mut self, reader: &schema::type_::tuple::Reader<'_>) -> Result<Arc<Type>> {
-        let elems = read_vec!(&reader.get_elements()?, self, type_);
-        Ok(Arc::new(Type::Tuple { elems }))
+        let elements = read_vec!(&reader.get_elements()?, self, type_);
+        Ok(Arc::new(Type::Tuple { elements }))
     }
 
     fn type_var(&mut self, reader: &schema::type_::var::Reader<'_>) -> Result<Arc<Type>> {
@@ -527,6 +577,7 @@ impl ModuleDecoder {
             location: self.src_span(&reader.get_location()?)?,
             literal: self.constant(&reader.get_literal()?)?,
             module: self.string(reader.get_module()?)?,
+            name: self.string(reader.get_name()?)?,
             implementations: self.implementations(reader.get_implementations()?),
         })
     }
