@@ -5,6 +5,7 @@ use crate::{
         Constant, Publicity, SrcSpan, TypedConstant, TypedConstantBitArraySegment,
         TypedConstantBitArraySegmentOption,
     },
+    reference::{Reference, ReferenceKind, ReferenceMap},
     schema_capnp::{self as schema, *},
     type_::{
         self, AccessorsMap, Deprecation, FieldMap, Opaque, RecordAccessor, Type,
@@ -52,6 +53,7 @@ impl<'a> ModuleEncoder<'a> {
         self.set_version(&mut module);
         self.set_module_documentation(&mut module);
         self.set_module_type_aliases(&mut module);
+        self.set_module_references(&mut module);
 
         capnp::serialize_packed::write_message(&mut buffer, &message).expect("capnp encode");
         Ok(buffer)
@@ -197,6 +199,56 @@ impl<'a> ModuleEncoder<'a> {
         }
     }
 
+    fn set_module_references(&mut self, module: &mut module::Builder<'_>) {
+        let references = &self.data.references;
+        let mut builder = module.reborrow().init_references();
+        let mut imported_modules = builder
+            .reborrow()
+            .init_imported_modules(references.imported_modules.len() as u32);
+        for (i, module) in references.imported_modules.iter().enumerate() {
+            imported_modules.set(i as u32, module);
+        }
+
+        let value_references = builder
+            .reborrow()
+            .init_value_references(references.value_references.len() as u32);
+        self.build_reference_map(value_references, &references.value_references);
+        let type_references = builder
+            .reborrow()
+            .init_type_references(references.type_references.len() as u32);
+        self.build_reference_map(type_references, &references.type_references);
+    }
+
+    fn build_reference_map(
+        &mut self,
+        mut builder: capnp::struct_list::Builder<'_, reference_map::Owned>,
+        map: &ReferenceMap,
+    ) {
+        for (i, ((module, name), references)) in map.iter().enumerate() {
+            let mut builder = builder.reborrow().get(i as u32);
+            builder.set_module(module);
+            builder.set_name(name);
+            let mut references_builder =
+                builder.reborrow().init_references(references.len() as u32);
+            for (i, reference) in references.iter().enumerate() {
+                let builder = references_builder.reborrow().get(i as u32);
+                self.build_reference(builder, reference);
+            }
+        }
+    }
+
+    fn build_reference(&mut self, mut builder: reference::Builder<'_>, reference: &Reference) {
+        let mut kind = builder.reborrow().init_kind();
+        match reference.kind {
+            ReferenceKind::Qualified => kind.set_qualified(()),
+            ReferenceKind::Unqualified => kind.set_unqualified(()),
+            ReferenceKind::Import => kind.set_import(()),
+            ReferenceKind::Definition => kind.set_definition(()),
+            ReferenceKind::Alias => kind.set_alias(()),
+        }
+        self.build_src_span(builder.init_location(), reference.location);
+    }
+
     fn set_version(&mut self, module: &mut module::Builder<'_>) {
         let mut version = module.reborrow().init_required_version();
         version.set_major(self.data.minimum_required_version.major);
@@ -336,12 +388,14 @@ impl<'a> ModuleEncoder<'a> {
                 module,
                 documentation: doc,
                 implementations,
+                name,
             } => {
                 let mut builder = builder.init_module_constant();
                 builder.set_documentation(doc.as_ref().map(EcoString::as_str).unwrap_or_default());
                 self.build_src_span(builder.reborrow().init_location(), *location);
                 self.build_constant(builder.reborrow().init_literal(), literal);
                 builder.reborrow().set_module(module);
+                builder.reborrow().set_name(name);
                 self.build_implementations(builder.init_implementations(), *implementations)
             }
 
@@ -558,10 +612,10 @@ impl<'a> ModuleEncoder<'a> {
 
     fn build_type(&mut self, builder: schema::type_::Builder<'_>, type_: &Type) {
         match type_ {
-            Type::Fn { args, retrn } => {
+            Type::Fn { args, return_ } => {
                 let mut fun = builder.init_fn();
                 self.build_types(fun.reborrow().init_arguments(args.len() as u32), args);
-                self.build_type(fun.init_return(), retrn)
+                self.build_type(fun.init_return(), return_)
             }
 
             Type::Named {
@@ -584,9 +638,9 @@ impl<'a> ModuleEncoder<'a> {
                 self.build_types(app.reborrow().init_parameters(args.len() as u32), args);
             }
 
-            Type::Tuple { elems } => self.build_types(
-                builder.init_tuple().init_elements(elems.len() as u32),
-                elems,
+            Type::Tuple { elements } => self.build_types(
+                builder.init_tuple().init_elements(elements.len() as u32),
+                elements,
             ),
 
             Type::Var { type_ } => match type_.borrow().deref() {
