@@ -10,10 +10,11 @@ use crate::{
     },
     build::Origin,
     line_numbers::LineNumbers,
+    reference::{Reference, ReferenceKind},
     type_::{
-        self, Deprecation, ModuleInterface, Opaque, Type, TypeAliasConstructor, TypeConstructor,
-        TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
-        ValueConstructorVariant, expression::Implementations, prelude,
+        self, Deprecation, ModuleInterface, Opaque, References, Type, TypeAliasConstructor,
+        TypeConstructor, TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors,
+        ValueConstructor, ValueConstructorVariant, expression::Implementations, prelude,
     },
     uid::UniqueIdGenerator,
 };
@@ -57,6 +58,7 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    name: "one".into(),
                 },
             },
         )]
@@ -67,6 +69,8 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     }
 }
 
@@ -104,6 +108,8 @@ fn empty_module() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -130,6 +136,8 @@ fn with_line_numbers() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -164,6 +172,8 @@ fn module_with_private_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -198,6 +208,8 @@ fn module_with_app_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -232,6 +244,8 @@ fn module_with_fn_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -266,6 +280,8 @@ fn module_with_tuple_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -306,6 +322,7 @@ fn module_with_generic_type() {
             type_aliases: HashMap::new(),
             documentation: Vec::new(),
             contains_echo: false,
+            references: References::default(),
         }
     }
 
@@ -346,6 +363,7 @@ fn module_with_type_links() {
             type_aliases: HashMap::new(),
             documentation: Vec::new(),
             contains_echo: false,
+            references: References::default(),
         }
     }
 
@@ -386,6 +404,7 @@ fn module_with_type_constructor_documentation() {
             type_aliases: HashMap::new(),
             documentation: Vec::new(),
             contains_echo: false,
+            references: References::default(),
         }
     }
 
@@ -429,6 +448,7 @@ fn module_with_type_constructor_origin() {
             type_aliases: HashMap::new(),
             documentation: Vec::new(),
             contains_echo: false,
+            references: References::default(),
         }
     }
 
@@ -465,6 +485,8 @@ fn module_type_to_constructors_mapping() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -516,6 +538,8 @@ fn module_fn_value() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -568,6 +592,8 @@ fn deprecated_module_fn_value() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -618,6 +644,8 @@ fn private_module_fn_value() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -670,6 +698,8 @@ fn module_fn_value_regression() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -721,6 +751,8 @@ fn module_fn_value_with_field_map() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -767,6 +799,8 @@ fn record_value() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -817,6 +851,8 @@ fn record_value_with_field_map() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -888,6 +924,8 @@ fn accessors() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -959,6 +997,7 @@ fn private_accessors() {
         minimum_required_version: Version::new(0, 1, 0),
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1120,6 +1159,7 @@ fn constant_var() {
                     can_run_on_erlang: true,
                     can_run_on_javascript: true,
                 },
+                name: "one_original".into(),
             },
         })),
     };
@@ -1152,6 +1192,7 @@ fn constant_var() {
                             can_run_on_erlang: true,
                             can_run_on_javascript: true,
                         },
+                        name: "one".into(),
                     },
                 },
             ),
@@ -1173,6 +1214,7 @@ fn constant_var() {
                             can_run_on_erlang: true,
                             can_run_on_javascript: true,
                         },
+                        name: "one_original".into(),
                     },
                 },
             ),
@@ -1184,6 +1226,8 @@ fn constant_var() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1379,6 +1423,8 @@ fn deprecated_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1429,6 +1475,7 @@ fn module_fn_value_with_external_implementations() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1452,6 +1499,7 @@ fn module_containing_echo() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: true,
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1505,6 +1553,8 @@ fn internal_module_fn() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1561,6 +1611,8 @@ fn internal_annotated_module_fn() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1609,6 +1661,8 @@ fn type_variable_ids_in_constructors_are_shared() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
 
     let expected = HashMap::from([(
@@ -1677,6 +1731,7 @@ fn type_with_inferred_variant() {
         minimum_required_version: Version::new(0, 1, 0),
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1711,6 +1766,8 @@ fn module_with_type_aliases() {
         .into(),
         documentation: Vec::new(),
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1737,6 +1794,8 @@ fn module_with_documentation() {
             "And finally, a third".into(),
         ],
         contains_echo: false,
+
+        references: References::default(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1771,6 +1830,97 @@ fn module_with_opaque_type() {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+        references: References::default(),
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn module_with_references() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        accessors: HashMap::new(),
+        values: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
+        references: References {
+            imported_modules: ["some_module".into(), "some_other_module".into()].into(),
+            value_references: [
+                (
+                    ("some_module".into(), "some_function".into()),
+                    vec![
+                        Reference {
+                            location: SrcSpan::new(1, 6),
+                            kind: ReferenceKind::Definition,
+                        },
+                        Reference {
+                            location: SrcSpan::new(7, 11),
+                            kind: ReferenceKind::Unqualified,
+                        },
+                    ],
+                ),
+                (
+                    ("some_other_module".into(), "some_constant".into()),
+                    vec![
+                        Reference {
+                            location: SrcSpan::new(6, 9),
+                            kind: ReferenceKind::Import,
+                        },
+                        Reference {
+                            location: SrcSpan::new(90, 108),
+                            kind: ReferenceKind::Unqualified,
+                        },
+                    ],
+                ),
+                (
+                    ("some_other_module".into(), "SomeTypeVariant".into()),
+                    vec![
+                        Reference {
+                            location: SrcSpan::new(26, 35),
+                            kind: ReferenceKind::Qualified,
+                        },
+                        Reference {
+                            location: SrcSpan::new(152, 204),
+                            kind: ReferenceKind::Qualified,
+                        },
+                        Reference {
+                            location: SrcSpan::new(0, 8),
+                            kind: ReferenceKind::Qualified,
+                        },
+                    ],
+                ),
+            ]
+            .into(),
+            type_references: [(
+                ("some_other_module".into(), "TypeVariant".into()),
+                vec![
+                    Reference {
+                        location: SrcSpan::new(26, 35),
+                        kind: ReferenceKind::Qualified,
+                    },
+                    Reference {
+                        location: SrcSpan::new(152, 204),
+                        kind: ReferenceKind::Unqualified,
+                    },
+                    Reference {
+                        location: SrcSpan::new(0, 8),
+                        kind: ReferenceKind::Definition,
+                    },
+                ],
+            )]
+            .into(),
+        },
     };
 
     assert_eq!(roundtrip(&module), module);
