@@ -659,6 +659,33 @@ pub fn one(x: Int) -> Int {
     );
 }
 
+#[test]
+fn unknown_target() {
+    assert_module_error!(
+        r#"
+@target(abc)
+pub fn one() {}"#
+    );
+}
+
+#[test]
+fn missing_target() {
+    assert_module_error!(
+        r#"
+@target()
+pub fn one() {}"#
+    );
+}
+
+#[test]
+fn missing_target_and_bracket() {
+    assert_module_error!(
+        r#"
+@target(
+pub fn one() {}"#
+    );
+}
+
 #[test]
 fn unknown_attribute() {
     assert_module_error!(
