@@ -166,6 +166,8 @@ fn paths() {
             .ends_with("hex/hexpm/packages/gleam_stdlib-0.17.1.tar")
     );
 
-    assert!(global_package_cache_package_tarball("elli", "1.0.0")
-        .ends_with("hex/hexpm/packages/elli-1.0.0.tar"));
+    assert!(
+        global_package_cache_package_tarball("elli", "1.0.0")
+            .ends_with("hex/hexpm/packages/elli-1.0.0.tar")
+    );
 }
