@@ -31,6 +31,7 @@ use crate::{
     bit_array,
     build::{Origin, Target},
     line_numbers::LineNumbers,
+    reference::ReferenceMap,
     type_::expression::Implementations,
 };
 use error::*;
@@ -97,7 +98,7 @@ pub enum Type {
     ///
     Fn {
         args: Vec<Arc<Type>>,
-        retrn: Arc<Type>,
+        return_: Arc<Type>,
     },
 
     /// A type variable. See the contained `TypeVar` enum for more information.
@@ -108,13 +109,13 @@ pub enum Type {
     /// can have a different type, so the `tuple` type is the sum of all the
     /// contained types.
     ///
-    Tuple { elems: Vec<Arc<Type>> },
+    Tuple { elements: Vec<Arc<Type>> },
 }
 
 impl Type {
     pub fn is_result_constructor(&self) -> bool {
         match self {
-            Type::Fn { retrn, .. } => retrn.is_result(),
+            Type::Fn { return_, .. } => return_.is_result(),
             Type::Var { type_ } => type_.borrow().is_result(),
             _ => false,
         }
@@ -173,7 +174,7 @@ impl Type {
 
     pub fn return_type(&self) -> Option<Arc<Self>> {
         match self {
-            Self::Fn { retrn, .. } => Some(retrn.clone()),
+            Self::Fn { return_, .. } => Some(return_.clone()),
             Self::Var { type_ } => type_.borrow().return_type(),
             _ => None,
         }
@@ -181,7 +182,7 @@ impl Type {
 
     pub fn fn_types(&self) -> Option<(Vec<Arc<Self>>, Arc<Self>)> {
         match self {
-            Self::Fn { args, retrn, .. } => Some((args.clone(), retrn.clone())),
+            Self::Fn { args, return_, .. } => Some((args.clone(), return_.clone())),
             Self::Var { type_ } => type_.borrow().fn_types(),
             _ => None,
         }
@@ -190,7 +191,7 @@ impl Type {
     /// Gets the types inside of a tuple. Returns `None` if the type is not a tuple.
     pub fn tuple_types(&self) -> Option<Vec<Arc<Self>>> {
         match self {
-            Self::Tuple { elems } => Some(elems.clone()),
+            Self::Tuple { elements } => Some(elements.clone()),
             Self::Var { type_, .. } => type_.borrow().tuple_types(),
             _ => None,
         }
@@ -230,6 +231,17 @@ impl Type {
         }
     }
 
+    pub fn list(inner_type: Arc<Self>) -> Self {
+        Type::Named {
+            publicity: Publicity::Public,
+            package: PRELUDE_PACKAGE_NAME.into(),
+            module: PRELUDE_MODULE_NAME.into(),
+            name: LIST.into(),
+            args: vec![inner_type],
+            inferred_variant: None,
+        }
+    }
+
     #[must_use]
     fn is_fun(&self) -> bool {
         match self {
@@ -327,16 +339,16 @@ impl Type {
                 inferred_variant, ..
             } => *inferred_variant = None,
             Type::Var { type_ } => type_.borrow_mut().generalise_custom_type_variant(),
-            Type::Tuple { elems } => {
-                for element in elems {
+            Type::Tuple { elements } => {
+                for element in elements {
                     Arc::make_mut(element).generalise_custom_type_variant();
                 }
             }
-            Type::Fn { args, retrn } => {
+            Type::Fn { args, return_ } => {
                 for argument in args {
                     Arc::make_mut(argument).generalise_custom_type_variant();
                 }
-                Arc::make_mut(retrn).generalise_custom_type_variant();
+                Arc::make_mut(return_).generalise_custom_type_variant();
             }
         }
     }
@@ -426,13 +438,15 @@ impl Type {
                 ..
             } => Some(self.clone()),
 
-            Self::Named { args, .. } => args.iter().find_map(|t| t.find_private_type()),
+            Self::Named { args, .. } => args.iter().find_map(|type_| type_.find_private_type()),
 
-            Self::Tuple { elems, .. } => elems.iter().find_map(|t| t.find_private_type()),
+            Self::Tuple { elements, .. } => {
+                elements.iter().find_map(|type_| type_.find_private_type())
+            }
 
-            Self::Fn { retrn, args, .. } => retrn
+            Self::Fn { return_, args, .. } => return_
                 .find_private_type()
-                .or_else(|| args.iter().find_map(|t| t.find_private_type())),
+                .or_else(|| args.iter().find_map(|type_| type_.find_private_type())),
 
             Self::Var { type_, .. } => match type_.borrow().deref() {
                 TypeVar::Unbound { .. } => None,
@@ -448,13 +462,15 @@ impl Type {
         match self {
             Self::Named { publicity, .. } if publicity.is_internal() => Some(self.clone()),
 
-            Self::Named { args, .. } => args.iter().find_map(|t| t.find_internal_type()),
+            Self::Named { args, .. } => args.iter().find_map(|type_| type_.find_internal_type()),
 
-            Self::Tuple { elems, .. } => elems.iter().find_map(|t| t.find_internal_type()),
+            Self::Tuple { elements, .. } => {
+                elements.iter().find_map(|type_| type_.find_internal_type())
+            }
 
-            Self::Fn { retrn, args, .. } => retrn
+            Self::Fn { return_, args, .. } => return_
                 .find_internal_type()
-                .or_else(|| args.iter().find_map(|t| t.find_internal_type())),
+                .or_else(|| args.iter().find_map(|type_| type_.find_internal_type())),
 
             Self::Var { type_, .. } => match type_.borrow().deref() {
                 TypeVar::Unbound { .. } | TypeVar::Generic { .. } => None,
@@ -514,10 +530,10 @@ impl Type {
                 type_.as_ref().borrow().same_as_other_type(one)
             }
             (
-                Type::Fn { args, retrn },
+                Type::Fn { args, return_ },
                 Type::Fn {
                     args: other_args,
-                    retrn: other_retrn,
+                    return_: other_return,
                 },
             ) => {
                 args.len() == other_args.len()
@@ -525,7 +541,7 @@ impl Type {
                         .iter()
                         .zip(other_args)
                         .all(|(one, other)| one.same_as(other))
-                    && retrn.same_as(other_retrn)
+                    && return_.same_as(other_return)
             }
 
             (Type::Var { type_ }, other) => type_.as_ref().borrow().same_as_other_type(other),
@@ -534,11 +550,16 @@ impl Type {
             (one @ Type::Tuple { .. }, Type::Var { type_ }) => {
                 type_.as_ref().borrow().same_as_other_type(one)
             }
-            (Type::Tuple { elems }, Type::Tuple { elems: other_elems }) => {
-                elems.len() == other_elems.len()
-                    && elems
+            (
+                Type::Tuple { elements },
+                Type::Tuple {
+                    elements: other_elements,
+                },
+            ) => {
+                elements.len() == other_elements.len()
+                    && elements
                         .iter()
-                        .zip(other_elems)
+                        .zip(other_elements)
                         .all(|(one, other)| one.same_as(other))
             }
         }
@@ -630,6 +651,7 @@ pub enum ValueConstructorVariant {
         documentation: Option<EcoString>,
         location: SrcSpan,
         module: EcoString,
+        name: EcoString,
         literal: Constant<Arc<Type>, EcoString>,
         implementations: Implementations,
     },
@@ -753,6 +775,22 @@ impl ValueConstructorVariant {
         matches!(self, Self::LocalVariable { .. })
     }
 
+    /// Returns `true` if the variant is a local variable generated by the compiler.
+    #[must_use]
+    pub fn is_generated_variable(&self) -> bool {
+        match self {
+            ValueConstructorVariant::LocalVariable {
+                origin: VariableOrigin::Generated,
+                ..
+            } => true,
+            ValueConstructorVariant::LocalVariable { .. } => false,
+            ValueConstructorVariant::ModuleConstant { .. }
+            | ValueConstructorVariant::LocalConstant { .. }
+            | ValueConstructorVariant::ModuleFn { .. }
+            | ValueConstructorVariant::Record { .. } => false,
+        }
+    }
+
     /// Returns `true` if the value constructor variant is [`ModuleFn`].
     ///
     /// [`ModuleFn`]: ValueConstructorVariant::ModuleFn
@@ -890,6 +928,7 @@ pub struct ModuleInterface {
     pub documentation: Vec<EcoString>,
     /// Wether there's any echo in the module.
     pub contains_echo: bool,
+    pub references: References,
 }
 
 impl ModuleInterface {
@@ -898,6 +937,13 @@ impl ModuleInterface {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+pub struct References {
+    pub imported_modules: HashSet<EcoString>,
+    pub value_references: ReferenceMap,
+    pub type_references: ReferenceMap,
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Opaque {
     Opaque,
@@ -1384,16 +1430,16 @@ fn unify_unbound_type(type_: Arc<Type>, own_id: u64) -> Result<(), UnifyError> {
             Ok(())
         }
 
-        Type::Fn { args, retrn } => {
+        Type::Fn { args, return_ } => {
             for arg in args {
                 unify_unbound_type(arg.clone(), own_id)?;
             }
-            unify_unbound_type(retrn.clone(), own_id)
+            unify_unbound_type(return_.clone(), own_id)
         }
 
-        Type::Tuple { elems, .. } => {
-            for elem in elems {
-                unify_unbound_type(elem.clone(), own_id)?
+        Type::Tuple { elements, .. } => {
+            for element in elements {
+                unify_unbound_type(element.clone(), own_id)?
             }
             Ok(())
         }
@@ -1415,31 +1461,31 @@ fn match_fun_type(
 
             TypeVar::Unbound { .. } => {
                 let args: Vec<_> = (0..arity).map(|_| environment.new_unbound_var()).collect();
-                let retrn = environment.new_unbound_var();
-                Some((args, retrn))
+                let return_ = environment.new_unbound_var();
+                Some((args, return_))
             }
 
             TypeVar::Generic { .. } => None,
         };
 
-        if let Some((args, retrn)) = new_value {
+        if let Some((args, return_)) = new_value {
             *type_.borrow_mut() = TypeVar::Link {
-                type_: fn_(args.clone(), retrn.clone()),
+                type_: fn_(args.clone(), return_.clone()),
             };
-            return Ok((args, retrn));
+            return Ok((args, return_));
         }
     }
 
-    if let Type::Fn { args, retrn } = type_.deref() {
+    if let Type::Fn { args, return_ } = type_.deref() {
         return if args.len() != arity {
             Err(MatchFunTypeError::IncorrectArity {
                 expected: args.len(),
                 given: arity,
                 args: args.clone(),
-                return_type: retrn.clone(),
+                return_type: return_.clone(),
             })
         } else {
-            Ok((args.clone(), retrn.clone()))
+            Ok((args.clone(), return_.clone()))
         };
     }
 
@@ -1464,7 +1510,7 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
             args,
             inferred_variant: _,
         } => {
-            let args = args.iter().map(|t| generalise(t.clone())).collect();
+            let args = args.iter().map(|type_| generalise(type_.clone())).collect();
             Arc::new(Type::Named {
                 publicity: *publicity,
                 module: module.clone(),
@@ -1475,12 +1521,17 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
             })
         }
 
-        Type::Fn { args, retrn } => fn_(
-            args.iter().map(|t| generalise(t.clone())).collect(),
-            generalise(retrn.clone()),
+        Type::Fn { args, return_ } => fn_(
+            args.iter().map(|type_| generalise(type_.clone())).collect(),
+            generalise(return_.clone()),
         ),
 
-        Type::Tuple { elems } => tuple(elems.iter().map(|t| generalise(t.clone())).collect()),
+        Type::Tuple { elements } => tuple(
+            elements
+                .iter()
+                .map(|type_| generalise(type_.clone()))
+                .collect(),
+        ),
     }
 }
 
