@@ -134,16 +134,10 @@ impl ModuleSuggestion {
         }
     }
 
-    pub fn name(&self) -> &EcoString {
-        match self {
-            ModuleSuggestion::Imported(name) | ModuleSuggestion::Importable(name) => name,
-        }
-    }
-
     pub fn last_name_component(&self) -> &str {
         match self {
             ModuleSuggestion::Imported(name) | ModuleSuggestion::Importable(name) => {
-                name.split('/').last().unwrap_or(name)
+                name.split('/').next_back().unwrap_or(name)
             }
         }
     }
@@ -605,6 +599,35 @@ pub enum Error {
     EchoWithNoFollowingExpression {
         location: SrcSpan,
     },
+    /// When someone tries concatenating two string values using the `+` operator.
+    ///
+    /// ```gleam
+    /// "aaa" + "bbb"
+    /// //    ^ We wont to suggest using `<>` instead!
+    /// ```
+    StringConcatenationWithAddInt {
+        location: SrcSpan,
+    },
+    /// When someone tries using an int operator on two floats.
+    ///
+    /// ```gleam
+    /// 1 +. 3
+    /// //^ We wont to suggest using `+` instead!
+    /// ```
+    FloatOperatorOnInts {
+        operator: BinOp,
+        location: SrcSpan,
+    },
+    /// When someone tries using an int operator on two floats.
+    ///
+    /// ```gleam
+    /// 1.2 + 1.0
+    /// //  ^ We wont to suggest using `+.` instead!
+    /// ```
+    IntOperatorOnFloats {
+        operator: BinOp,
+        location: SrcSpan,
+    },
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -870,6 +893,10 @@ pub enum Warning {
         location: SrcSpan,
     },
 
+    AssertAssignmentOnInferredVariant {
+        location: SrcSpan,
+    },
+
     /// When a `todo` or `panic` is used as a function instead of providing the
     /// error message with the `as` syntax.
     ///
@@ -932,6 +959,7 @@ pub enum FeatureKind {
     ConstantStringConcatenation,
     ArithmeticInGuards,
     UnannotatedUtf8StringSegment,
+    UnannotatedFloatSegment,
     NestedTupleAccess,
     InternalAnnotation,
     AtInJavascriptModules,
@@ -967,6 +995,7 @@ impl FeatureKind {
             }
 
             FeatureKind::JavaScriptUnalignedBitArray => Version::new(1, 9, 0),
+            FeatureKind::UnannotatedFloatSegment => Version::new(1, 10, 0),
         }
     }
 }
@@ -1072,7 +1101,10 @@ impl Error {
             | Error::AllVariantsDeprecated { location }
             | Error::EchoWithNoFollowingExpression { location }
             | Error::DeprecatedVariantOnDeprecatedType { location }
-            | Error::ErlangFloatUnsafe { location } => location.start,
+            | Error::ErlangFloatUnsafe { location }
+            | Error::FloatOperatorOnInts { location, .. }
+            | Error::IntOperatorOnFloats { location, .. }
+            | Error::StringConcatenationWithAddInt { location } => location.start,
 
             Error::UnknownLabels { unknown, .. } => {
                 unknown.iter().map(|(_, s)| s.start).min().unwrap_or(0)
@@ -1131,6 +1163,7 @@ impl Warning {
             | Warning::OpaqueExternalType { location, .. }
             | Warning::InternalTypeLeak { location, .. }
             | Warning::RedundantAssertAssignment { location, .. }
+            | Warning::AssertAssignmentOnInferredVariant { location, .. }
             | Warning::TodoOrPanicUsedAsFunction { location, .. }
             | Warning::UnreachableCodeAfterPanic { location, .. }
             | Warning::RedundantPipeFunctionCapture { location, .. }
