@@ -12,6 +12,7 @@ use crate::{
     ast::{
         AssignName, BitArrayOption, ImplicitCallArgOrigin, Layer, UntypedPatternBitArraySegment,
     },
+    reference::ReferenceKind,
     type_::expression::FunctionDefinition,
 };
 use std::sync::Arc;
@@ -70,14 +71,8 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         match &mut self.mode {
             PatternMode::Initial => {
                 // Register usage for the unused variable detection
-                self.environment.init_usage(
-                    name.into(),
-                    EntityKind::Variable {
-                        origin: origin.clone(),
-                    },
-                    location,
-                    self.problems,
-                );
+                self.environment
+                    .init_usage(name.into(), origin.clone(), location, self.problems);
                 // Ensure there are no duplicate variable names in the pattern
                 if self.initial_pattern_vars.contains(name) {
                     return Err(UnifyError::DuplicateVarInPattern { name: name.into() });
@@ -101,8 +96,8 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     // This variable was defined in the Initial multi-pattern
                     Some(initial) if self.initial_pattern_vars.contains(name) => {
                         assigned.push(name.into());
-                        let initial_typ = initial.type_.clone();
-                        unify(initial_typ, type_.clone())?;
+                        let initial_type = initial.type_.clone();
+                        unify(initial_type, type_.clone())?;
                         unify_constructor_variants(Arc::make_mut(&mut initial.type_), &type_);
                         Ok(())
                     }
@@ -266,27 +261,35 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
 
     fn infer_pattern_segment(
         &mut self,
-        segment: UntypedPatternBitArraySegment,
+        mut segment: UntypedPatternBitArraySegment,
         is_last_segment: bool,
     ) -> Result<TypedPatternBitArraySegment, Error> {
-        let UntypedPatternBitArraySegment {
-            location,
-            options,
-            value,
-            ..
-        } = segment;
+        // If the segment doesn't have an explicit type option we add a default
+        // one ourselves if the pattern is unambiguous: literal strings are
+        // implicitly considered utf-8 encoded strings, while floats are
+        // implicitly given the float type option.
+        if !segment.has_type_option() {
+            match segment.value.as_ref() {
+                Pattern::String { location, .. } => {
+                    self.track_feature_usage(FeatureKind::UnannotatedUtf8StringSegment, *location);
+                    segment.options.push(BitArrayOption::Utf8 {
+                        location: SrcSpan::default(),
+                    });
+                }
 
-        let options = match value.as_ref() {
-            Pattern::String { location, .. } if options.is_empty() => {
-                self.track_feature_usage(FeatureKind::UnannotatedUtf8StringSegment, *location);
-                vec![BitArrayOption::Utf8 {
-                    location: SrcSpan::default(),
-                }]
+                Pattern::Float { location, .. } => {
+                    self.track_feature_usage(FeatureKind::UnannotatedFloatSegment, *location);
+                    segment.options.push(BitArrayOption::Float {
+                        location: SrcSpan::default(),
+                    })
+                }
+
+                _ => (),
             }
-            _ => options,
-        };
+        }
 
-        let options: Vec<_> = options
+        let options: Vec<_> = segment
+            .options
             .into_iter()
             .map(|o| {
                 crate::analyse::infer_bit_array_option(o, |value, type_| {
@@ -339,20 +342,20 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         }
 
         let type_ = {
-            match value.deref() {
+            match segment.value.deref() {
                 Pattern::Variable { .. } if segment_type == string() => {
                     Err(Error::BitArraySegmentError {
                         error: bit_array::ErrorType::VariableUtfSegmentInPattern,
-                        location,
+                        location: segment.location,
                     })
                 }
                 _ => Ok(segment_type),
             }
         }?;
-        let typed_value = self.unify(*value, type_.clone(), None)?;
+        let typed_value = self.unify(*segment.value, type_.clone(), None)?;
 
         Ok(BitArraySegment {
-            location,
+            location: segment.location,
             value: Box::new(typed_value),
             options,
             type_,
@@ -593,46 +596,48 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 }),
             },
 
-            Pattern::Tuple { elems, location } => match collapse_links(type_.clone()).deref() {
-                Type::Tuple { elems: type_elems } => {
-                    if elems.len() != type_elems.len() {
+            Pattern::Tuple { elements, location } => match collapse_links(type_.clone()).deref() {
+                Type::Tuple {
+                    elements: type_elements,
+                } => {
+                    if elements.len() != type_elements.len() {
                         return Err(Error::IncorrectArity {
                             labels: vec![],
                             location,
-                            expected: type_elems.len(),
-                            given: elems.len(),
+                            expected: type_elements.len(),
+                            given: elements.len(),
                         });
                     }
 
-                    let elems = elems
+                    let elements = elements
                         .into_iter()
-                        .zip(type_elems)
+                        .zip(type_elements)
                         .map(|(pattern, type_)| self.unify(pattern, type_.clone(), None))
                         .try_collect()?;
-                    Ok(Pattern::Tuple { elems, location })
+                    Ok(Pattern::Tuple { elements, location })
                 }
 
                 Type::Var { .. } => {
-                    let elems_types: Vec<_> = (0..(elems.len()))
+                    let elements_types: Vec<_> = (0..(elements.len()))
                         .map(|_| self.environment.new_unbound_var())
                         .collect();
-                    unify(tuple(elems_types.clone()), type_)
+                    unify(tuple(elements_types.clone()), type_)
                         .map_err(|e| convert_unify_error(e, location))?;
-                    let elems = elems
+                    let elements = elements
                         .into_iter()
-                        .zip(elems_types)
+                        .zip(elements_types)
                         .map(|(pattern, type_)| self.unify(pattern, type_, None))
                         .try_collect()?;
-                    Ok(Pattern::Tuple { elems, location })
+                    Ok(Pattern::Tuple { elements, location })
                 }
 
                 _ => {
-                    let elems_types = (0..(elems.len()))
+                    let elements_types = (0..(elements.len()))
                         .map(|_| self.environment.new_unbound_var())
                         .collect();
 
                     Err(Error::CouldNotUnify {
-                        given: tuple(elems_types),
+                        given: tuple(elements_types),
                         expected: type_,
                         situation: None,
                         location,
@@ -648,6 +653,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
             Pattern::Constructor {
                 location,
                 module,
+                name_location,
                 name,
                 arguments: mut pattern_args,
                 spread,
@@ -656,7 +662,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 // Register the value as seen for detection of unused values
                 self.environment.increment_usage(&name);
 
-                let cons = self
+                let constructor = self
                     .environment
                     .get_value_constructor(module.as_ref().map(|(module, _)| module), &name)
                     .map_err(|e| {
@@ -667,7 +673,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                         )
                     })?;
 
-                match cons.field_map() {
+                match constructor.field_map() {
                     // The fun has a field map so labelled arguments may be present and need to be reordered.
                     Some(field_map) => {
                         if let Some(spread_location) = spread {
@@ -688,7 +694,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             // Potential future optimisation.
                             let index_of_first_labelled_arg = pattern_args
                                 .iter()
-                                .position(|a| a.label.is_some())
+                                .position(|argument| argument.label.is_some())
                                 .unwrap_or(pattern_args.len());
 
                             // In Gleam we can pass in positional unlabelled args to a constructor
@@ -716,7 +722,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             let supplied_unlabelled_args = index_of_first_labelled_arg;
                             let supplied_labelled_args = pattern_args
                                 .iter()
-                                .filter_map(|l| l.label.clone())
+                                .filter_map(|argument| argument.label.clone())
                                 .collect::<HashSet<_>>();
                             let constructor_unlabelled_args =
                                 field_map.arity - field_map.fields.len() as u32;
@@ -728,7 +734,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                                 .fields
                                 .iter()
                                 // We take the labels in order of definition in the constructor...
-                                .sorted_by_key(|(_, pos)| *pos)
+                                .sorted_by_key(|(_, position)| *position)
                                 .map(|(label, _)| label.clone())
                                 // ...and then remove the ones that were supplied as unlabelled
                                 // positional arguments...
@@ -761,7 +767,9 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                         assert_no_labelled_arguments(&pattern_args)?;
 
                         if let Some(spread_location) = spread {
-                            if let ValueConstructorVariant::Record { arity, .. } = &cons.variant {
+                            if let ValueConstructorVariant::Record { arity, .. } =
+                                &constructor.variant
+                            {
                                 while pattern_args.len() < usize::from(*arity) {
                                     pattern_args.push(CallArg {
                                         value: Pattern::Discard {
@@ -779,8 +787,9 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     }
                 }
 
-                let constructor_typ = cons.type_.clone();
-                let constructor = match &cons.variant {
+                let constructor_type = constructor.type_.clone();
+                let constructor_deprecation = constructor.deprecation.clone();
+                let pattern_constructor = match &constructor.variant {
                     ValueConstructorVariant::Record {
                         name,
                         documentation,
@@ -788,14 +797,26 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                         location,
                         variant_index: constructor_index,
                         ..
-                    } => PatternConstructor {
-                        documentation: documentation.clone(),
-                        name: name.clone(),
-                        field_map: cons.field_map().cloned(),
-                        module: module.clone(),
-                        location: *location,
-                        constructor_index: *constructor_index,
-                    },
+                    } => {
+                        let constructor_index = *constructor_index;
+                        let constructor = PatternConstructor {
+                            documentation: documentation.clone(),
+                            name: name.clone(),
+                            field_map: constructor.field_map().cloned(),
+                            module: module.clone(),
+                            location: *location,
+                            constructor_index,
+                        };
+
+                        if let Some(ref variable_name) = subject_variable {
+                            self.set_subject_variable_variant(
+                                variable_name.clone(),
+                                constructor_index,
+                            );
+                        }
+
+                        constructor
+                    }
                     ValueConstructorVariant::LocalVariable { .. }
                     | ValueConstructorVariant::LocalConstant { .. }
                     | ValueConstructorVariant::ModuleConstant { .. }
@@ -804,7 +825,6 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     }
                 };
 
-                let constructor_deprecation = cons.deprecation.clone();
                 match constructor_deprecation {
                     Deprecation::NotDeprecated => {}
                     Deprecation::Deprecated { message } => {
@@ -816,11 +836,23 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     }
                 }
 
+                self.environment.references.register_value_reference(
+                    pattern_constructor.module.clone(),
+                    pattern_constructor.name.clone(),
+                    &name,
+                    name_location,
+                    if module.is_some() {
+                        ReferenceKind::Qualified
+                    } else {
+                        ReferenceKind::Unqualified
+                    },
+                );
+
                 let instantiated_constructor_type =
                     self.environment
-                        .instantiate(constructor_typ, &mut hashmap![], self.hydrator);
+                        .instantiate(constructor_type, &mut hashmap![], self.hydrator);
                 match instantiated_constructor_type.deref() {
-                    Type::Fn { args, retrn } => {
+                    Type::Fn { args, return_ } => {
                         if args.len() == pattern_args.len() {
                             let pattern_args = pattern_args
                                 .into_iter()
@@ -848,11 +880,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                                     })
                                 })
                                 .try_collect()?;
-                            unify(type_.clone(), retrn.clone())
+                            unify(type_.clone(), return_.clone())
                                 .map_err(|e| convert_unify_error(e, location))?;
 
                             if let Some((variable_to_infer, inferred_variant)) =
-                                subject_variable.zip(retrn.custom_type_inferred_variant())
+                                subject_variable.zip(return_.custom_type_inferred_variant())
                             {
                                 self.set_subject_variable_variant(
                                     variable_to_infer,
@@ -862,12 +894,13 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
 
                             Ok(Pattern::Constructor {
                                 location,
+                                name_location,
                                 module,
                                 name,
                                 arguments: pattern_args,
-                                constructor: Inferred::Known(constructor),
+                                constructor: Inferred::Known(pattern_constructor),
                                 spread,
-                                type_: retrn.clone(),
+                                type_: return_.clone(),
                             })
                         } else {
                             Err(Error::IncorrectArity {
@@ -897,10 +930,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
 
                             Ok(Pattern::Constructor {
                                 location,
+                                name_location,
                                 module,
                                 name,
                                 arguments: vec![],
-                                constructor: Inferred::Known(constructor),
+                                constructor: Inferred::Known(pattern_constructor),
                                 spread,
                                 type_: instantiated_constructor_type,
                             })
