@@ -9,7 +9,7 @@ use crate::{
 };
 
 use super::{
-    ModuleInterface, Opaque, Type, TypeConstructor, TypeValueConstructor,
+    ModuleInterface, Opaque, References, Type, TypeConstructor, TypeValueConstructor,
     TypeValueConstructorField, TypeVar, TypeVariantConstructors, ValueConstructor,
     ValueConstructorVariant,
 };
@@ -153,12 +153,12 @@ fn result_with_variant(a: Arc<Type>, e: Arc<Type>, variant_index: Option<u16>) -
     })
 }
 
-pub fn tuple(elems: Vec<Arc<Type>>) -> Arc<Type> {
-    Arc::new(Type::Tuple { elems })
+pub fn tuple(elements: Vec<Arc<Type>>) -> Arc<Type> {
+    Arc::new(Type::Tuple { elements })
 }
 
-pub fn fn_(args: Vec<Arc<Type>>, retrn: Arc<Type>) -> Arc<Type> {
-    Arc::new(Type::Fn { retrn, args })
+pub fn fn_(args: Vec<Arc<Type>>, return_: Arc<Type>) -> Arc<Type> {
+    Arc::new(Type::Fn { return_, args })
 }
 
 pub fn named(
@@ -245,6 +245,7 @@ pub fn build_prelude(ids: &UniqueIdGenerator) -> ModuleInterface {
         type_aliases: HashMap::new(),
         documentation: Vec::new(),
         contains_echo: false,
+        references: References::default(),
     };
 
     for t in PreludeType::iter() {
