@@ -162,6 +162,10 @@ pub enum Warning {
         src: EcoString,
         warning: DeprecatedSyntaxWarning,
     },
+
+    DeprecatedEnvironmentVariable {
+        name: String,
+    },
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Copy)]
@@ -354,7 +358,7 @@ To match on all possible lists, use the `_` catch-all pattern instead.",
                 }
             }
 
-            Self::Type { path, warning, src } => match warning {
+            Warning::Type { path, warning, src } => match warning {
                 type_::Warning::Todo {
                     kind,
                     location,
@@ -940,6 +944,27 @@ hidden from the package's documentation.",
                     }),
                 },
 
+                type_::Warning::AssertAssignmentOnInferredVariant { location } => Diagnostic {
+                    title: "Assertion that will always fail".into(),
+                    text: wrap(
+                        "We can tell from the code above that the value will never match \
+this pattern and that this code will always crash.
+
+Either change the pattern or use `panic` to unconditionally fail.",
+                    ),
+                    hint: None,
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        label: diagnostic::Label {
+                            text: None,
+                            span: *location,
+                        },
+                        path: path.clone(),
+                        src: src.clone(),
+                        extra_labels: vec![],
+                    }),
+                },
+
                 type_::Warning::TodoOrPanicUsedAsFunction {
                     kind,
                     location,
@@ -1071,6 +1096,9 @@ See: https://tour.gleam.run/functions/pipelines/",
                         FeatureKind::UnannotatedUtf8StringSegment => {
                             "The ability to omit the `utf8` annotation for string segments was"
                         }
+                        FeatureKind::UnannotatedFloatSegment => {
+                            "The ability to omit the `float` annotation for float segments was"
+                        }
                         FeatureKind::NestedTupleAccess => {
                             "The ability to access nested tuple fields was"
                         }
@@ -1145,6 +1173,14 @@ information.",
                     }),
                 },
             },
+
+            Warning::DeprecatedEnvironmentVariable { name } => Diagnostic {
+                title: "Use of deprecated environment variable".into(),
+                text: wrap(&format!("The environment variable `{name}` is deprecated.")),
+                hint: None,
+                level: diagnostic::Level::Warning,
+                location: None,
+            },
         }
     }
 
