@@ -3,10 +3,18 @@ use gleam_core::build::Telemetry;
 pub struct LogTelemetry;
 
 impl Telemetry for LogTelemetry {
+    fn compiled_package(&self, duration: std::time::Duration) {
+        tracing::info!("Compiled in {} ", seconds(duration));
+    }
+
     fn compiling_package(&self, name: &str) {
         tracing::info!("Compiling package: {}", name);
     }
 
+    fn checked_package(&self, duration: std::time::Duration) {
+        tracing::info!("Checked in {}", seconds(duration));
+    }
+
     fn checking_package(&self, name: &str) {
         tracing::info!("Checking package: {}", name);
     }
@@ -15,6 +23,10 @@ impl Telemetry for LogTelemetry {
         tracing::info!("Downloading package: {}", name);
     }
 
+    fn running(&self, name: &str) {
+        tracing::info!("Running {}", name);
+    }
+
     fn resolving_package_versions(&self) {
         tracing::info!("Resolving package versions");
     }
@@ -27,3 +39,7 @@ impl Telemetry for LogTelemetry {
         tracing::info!("Waiting for build directory lock");
     }
 }
+
+pub fn seconds(duration: std::time::Duration) -> String {
+    format!("{:.2}s", duration.as_millis() as f32 / 1000.)
+}
